@@ -1,11 +1,19 @@
-# Copyright (c) 2016 by Ron Frederick <ronf@timeheart.net>.
-# All rights reserved.
+# Copyright (c) 2016-2019 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
-# the terms of the Eclipse Public License v1.0 which accompanies this
+# the terms of the Eclipse Public License v2.0 which accompanies this
 # distribution and is available at:
 #
-#     http://www.eclipse.org/legal/epl-v10.html
+#     http://www.eclipse.org/legal/epl-2.0/
+#
+# This program may also be made available under the following secondary
+# licenses when the conditions for such availability set forth in the
+# Eclipse Public License v2.0 are satisfied:
+#
+#    GNU General Public License, Version 2.0, or any later versions of
+#    that license
+#
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 #
 # Contributors:
 #     Ron Frederick - initial implementation, API, and documentation
@@ -16,11 +24,10 @@ import asyncio
 import errno
 import os
 import sys
-import tempfile
 
-import asyncssh
-
-from .logging import logger
+from .misc import async_context_manager, maybe_wait_closed
+from .packet import Byte, String, UInt32, PacketDecodeError, SSHPacket
+from .public_key import SSHKeyPair, load_default_keypairs, load_keypairs
 
 
 try:
@@ -29,18 +36,11 @@ try:
     else:
         from .agent_unix import open_agent
 except ImportError as exc: # pragma: no cover
-    def open_agent(loop, agent_path, reason=str(exc)):
+    async def open_agent(_agent_path, reason=str(exc)):
         """Dummy function if we're unable to import agent support"""
 
-        # pylint: disable=unused-argument
-
         raise OSError(errno.ENOENT, 'Agent support unavailable: %s' % reason)
 
-from .listener import create_unix_forward_listener
-from .misc import ChannelOpenError, load_default_keypairs
-from .packet import Byte, String, UInt32, PacketDecodeError, SSHPacket
-from .public_key import SSHKeyPair
-
 
 # pylint: disable=bad-whitespace
 
@@ -68,7 +68,7 @@ SSH_AGENT_EXTENSION_FAILURE              = 28
 # SSH agent constraint numbers
 SSH_AGENT_CONSTRAIN_LIFETIME             = 1
 SSH_AGENT_CONSTRAIN_CONFIRM              = 2
-SSH_AGENT_CONSTRAIN_EXTENSION            = 3
+SSH_AGENT_CONSTRAIN_EXTENSION            = 255
 
 # SSH agent signature flags
 SSH_AGENT_RSA_SHA2_256                   = 2
@@ -77,56 +77,58 @@ SSH_AGENT_RSA_SHA2_512                   = 4
 # pylint: enable=bad-whitespace
 
 
-class _X11AgentListener:
-    """Listener used to forward agent connections"""
-
-    def __init__(self, tempdir, path, unix_listener):
-        self._tempdir = tempdir
-        self._path = path
-        self._unix_listener = unix_listener
-
-    def get_path(self):
-        """Return the path being listened on"""
-
-        return self._path
-
-    def close(self):
-        """Close the agent listener"""
-
-        self._unix_listener.close()
-        self._tempdir.cleanup()
-
-
 class SSHAgentKeyPair(SSHKeyPair):
     """Surrogate for a key managed by the SSH agent"""
 
     _key_type = 'agent'
 
-    def __init__(self, agent, algorithm, public_data, comment):
-        super().__init__(algorithm, comment)
+    def __init__(self, agent, algorithm, key_blob, comment):
+        super().__init__(algorithm, key_blob, comment)
 
         self._agent = agent
-        self.public_data = public_data
+        self.key_blob = key_blob
+        self.sig_algorithm = algorithm
+        self.sig_algorithms = (algorithm,)
+        self.host_key_algorithms = self.sig_algorithms
 
         self._cert = algorithm.endswith(b'-cert-v01@openssh.com')
         self._flags = 0
 
-        if self._cert:
-            self.sig_algorithm = algorithm[:-21]
-        else:
-            self.sig_algorithm = algorithm
+        self._set_algorithms()
 
-        if self.sig_algorithm == b'ssh-rsa':
+    def _set_algorithms(self):
+        """Set the algorithms associated with this key"""
+
+        if self._cert:
+            if self.algorithm.startswith(b'sk-'):
+                self.sig_algorithm = self.algorithm[:-21] + b'@openssh.com'
+            else:
+                self.sig_algorithm = self.algorithm[:-21]
+
+        # Neither Pageant nor the Win10 OpenSSH agent seems to support the
+        # ssh-agent protocol flags used to request RSA SHA2 signatures yet
+        if self.sig_algorithm == b'ssh-rsa' and sys.platform != 'win32':
             self.sig_algorithms = (b'rsa-sha2-256', b'rsa-sha2-512',
                                    b'ssh-rsa')
         else:
             self.sig_algorithms = (self.sig_algorithm,)
 
         if self._cert:
-            self.host_key_algorithms = (algorithm,)
+            self.host_key_algorithms = (self.algorithm,)
         else:
             self.host_key_algorithms = self.sig_algorithms
 
+    def set_certificate(self, cert):
+        """Set certificate to use with this key"""
+
+        if self.key_blob != cert.key.public_data:
+            raise ValueError('Certificate key mismatch')
+
+        self._cert = True
+        self.algorithm = cert.algorithm
+        self.public_data = cert.public_data
+        self._set_algorithms()
+
     def set_sig_algorithm(self, sig_algorithm):
         """Set the signature algorithm to use when signing data"""
 
@@ -140,37 +142,41 @@ class SSHAgentKeyPair(SSHKeyPair):
         elif sig_algorithm == b'rsa-sha2-512':
             self._flags |= SSH_AGENT_RSA_SHA2_512
 
-    @asyncio.coroutine
-    def sign(self, data):
+    async def sign(self, data):
         """Sign a block of data with this private key"""
 
-        return (yield from self._agent.sign(self.public_data,
-                                            data, self._flags))
+        return await self._agent.sign(self.key_blob, data, self._flags)
 
-    @asyncio.coroutine
-    def remove(self):
+    async def remove(self):
         """Remove this key pair from the agent"""
 
-        yield from self._agent.remove_keys([self])
+        await self._agent.remove_keys([self])
 
 
 class SSHAgentClient:
     """SSH agent client"""
 
-    def __init__(self, loop, agent_path):
-        self._loop = loop
+    def __init__(self, agent_path):
         self._agent_path = agent_path
         self._reader = None
         self._writer = None
-        self._lock = asyncio.Lock(loop=loop)
+        self._lock = asyncio.Lock()
 
-    def _cleanup(self):
+    async def __aenter__(self):
+        """Allow SSHAgentClient to be used as an async context manager"""
+
+        return self
+
+    async def __aexit__(self, *exc_info):
+        """Wait for connection close when used as an async context manager"""
+
+        await self._cleanup()
+
+    async def _cleanup(self):
         """Clean up this SSH agent client"""
 
-        if self._writer:
-            self._writer.close()
-            self._reader = None
-            self._writer = None
+        self.close()
+        await self.wait_closed()
 
     @staticmethod
     def encode_constraints(lifetime, confirm):
@@ -186,44 +192,40 @@ class SSHAgentClient:
 
         return result
 
-    @asyncio.coroutine
-    def connect(self):
+    async def connect(self):
         """Connect to the SSH agent"""
 
-        if isinstance(self._agent_path, asyncssh.SSHServerConnection):
+        if hasattr(self._agent_path, 'open_agent_connection'):
             self._reader, self._writer = \
-                yield from self._agent_path.open_agent_connection()
+                await self._agent_path.open_agent_connection()
         else:
-            self._reader, self._writer = \
-                yield from open_agent(self._loop, self._agent_path)
+            self._reader, self._writer = await open_agent(self._agent_path)
 
-    @asyncio.coroutine
-    def _make_request(self, msgtype, *args):
+    async def _make_request(self, msgtype, *args):
         """Send an SSH agent request"""
 
-        with (yield from self._lock):
+        async with self._lock:
             try:
                 if not self._writer:
-                    yield from self.connect()
+                    await self.connect()
 
                 payload = Byte(msgtype) + b''.join(args)
                 self._writer.write(UInt32(len(payload)) + payload)
 
-                resplen = yield from self._reader.readexactly(4)
+                resplen = await self._reader.readexactly(4)
                 resplen = int.from_bytes(resplen, 'big')
 
-                resp = yield from self._reader.readexactly(resplen)
+                resp = await self._reader.readexactly(resplen)
                 resp = SSHPacket(resp)
 
                 resptype = resp.get_byte()
 
                 return resptype, resp
             except (OSError, EOFError, PacketDecodeError) as exc:
-                self._cleanup()
+                await self._cleanup()
                 raise ValueError(str(exc)) from None
 
-    @asyncio.coroutine
-    def get_keys(self):
+    async def get_keys(self):
         """Request the available client keys
 
            This method is a coroutine which returns a list of client keys
@@ -234,7 +236,7 @@ class SSHAgentClient:
         """
 
         resptype, resp = \
-            yield from self._make_request(SSH_AGENTC_REQUEST_IDENTITIES)
+            await self._make_request(SSH_AGENTC_REQUEST_IDENTITIES)
 
         if resptype == SSH_AGENT_IDENTITIES_ANSWER:
             result = []
@@ -255,14 +257,12 @@ class SSHAgentClient:
         else:
             raise ValueError('Unknown SSH agent response: %d' % resptype)
 
-    @asyncio.coroutine
-    def sign(self, key_blob, data, flags=0):
+    async def sign(self, key_blob, data, flags=0):
         """Sign a block of data with the requested key"""
 
-        resptype, resp = \
-            yield from self._make_request(SSH_AGENTC_SIGN_REQUEST,
-                                          String(key_blob), String(data),
-                                          UInt32(flags))
+        resptype, resp = await self._make_request(SSH_AGENTC_SIGN_REQUEST,
+                                                  String(key_blob),
+                                                  String(data), UInt32(flags))
 
         if resptype == SSH_AGENT_SIGN_RESPONSE:
             sig = resp.get_string()
@@ -273,9 +273,8 @@ class SSHAgentClient:
         else:
             raise ValueError('Unknown SSH agent response: %d' % resptype)
 
-    @asyncio.coroutine
-    def add_keys(self, keylist=(), passphrase=None,
-                 lifetime=None, confirm=False):
+    async def add_keys(self, keylist=(), passphrase=None,
+                       lifetime=None, confirm=False):
         """Add keys to the agent
 
            This method adds a list of local private keys and optional
@@ -283,71 +282,96 @@ class SSHAgentClient:
 
            :param keylist: (optional)
                The list of keys to add. If not specified, an attempt will
-               be made to load keys from the files :file:`.ssh/id_ed25519`,
+               be made to load keys from the files
+               :file:`.ssh/id_ed25519_sk`, :file:`.ssh/id_ecdsa_sk`,
+               :file:`.ssh/id_ed448`, :file:`.ssh/id_ed25519`,
                :file:`.ssh/id_ecdsa`, :file:`.ssh/id_rsa` and
                :file:`.ssh/id_dsa` in the user's home directory with
                optional matching certificates loaded from the files
+               :file:`.ssh/id_ed25519_sk-cert.pub`,
+               :file:`.ssh/id_ecdsa_sk-cert.pub`,
+               :file:`.ssh/id_ed448-cert.pub`,
                :file:`.ssh/id_ed25519-cert.pub`,
                :file:`.ssh/id_ecdsa-cert.pub`, :file:`.ssh/id_rsa-cert.pub`,
-               and :file:`.ssh/id_dsa-cert.pub`.
-           :param str passphrase: (optional)
+               and :file:`.ssh/id_dsa-cert.pub`. Failures when adding keys
+               are ignored in this case, as the agent may not recognize
+               some of these key types.
+           :param passphrase: (optional)
                The passphrase to use to decrypt the keys.
            :param lifetime: (optional)
                The time in seconds after which the keys should be
-               automatically deleted, or ``None`` to store these keys
+               automatically deleted, or `None` to store these keys
                indefinitely (the default).
-           :param bool confirm: (optional)
+           :param confirm: (optional)
                Whether or not to require confirmation for each private
-               key operation which uses these keys, defaulting to ``False``.
+               key operation which uses these keys, defaulting to `False`.
            :type keylist: *see* :ref:`SpecifyingPrivateKeys`
-           :type lifetime: `int` or ``None``
+           :type passphrase: `str`
+           :type lifetime: `int` or `None`
+           :type confirm: `bool`
 
            :raises: :exc:`ValueError` if the keys cannot be added
 
         """
 
         if keylist:
-            keypairs = asyncssh.load_keypairs(keylist, passphrase)
+            keypairs = load_keypairs(keylist, passphrase)
+            ignore_failures = False
         else:
             keypairs = load_default_keypairs(passphrase)
+            ignore_failures = True
 
-        constraints = self.encode_constraints(lifetime, confirm)
-        msgtype = SSH_AGENTC_ADD_ID_CONSTRAINED if constraints else \
-                      SSH_AGENTC_ADD_IDENTITY
+        base_constraints = self.encode_constraints(lifetime, confirm)
+
+        provider = os.environ.get('SSH_SK_PROVIDER') or 'internal'
+
+        sk_constraints = Byte(SSH_AGENT_CONSTRAIN_EXTENSION) + \
+                         String('sk-provider@openssh.com') + \
+                         String(provider)
 
         for keypair in keypairs:
-            comment = keypair.get_comment()
+            constraints = base_constraints
+
+            if keypair.algorithm.startswith(b'sk-'):
+                constraints += sk_constraints
+
+            msgtype = SSH_AGENTC_ADD_ID_CONSTRAINED if constraints else \
+                          SSH_AGENTC_ADD_IDENTITY
+
+            comment = keypair.get_comment_bytes()
+
             resptype, resp = \
-                yield from self._make_request(msgtype,
-                                              keypair.get_agent_private_key(),
-                                              String(comment or ''),
-                                              constraints)
+                await self._make_request(msgtype,
+                                         keypair.get_agent_private_key(),
+                                         String(comment or b''), constraints)
 
             if resptype == SSH_AGENT_SUCCESS:
                 resp.check_end()
             elif resptype == SSH_AGENT_FAILURE:
-                raise ValueError('Unable to add key')
+                if not ignore_failures:
+                    raise ValueError('Unable to add key')
             else:
                 raise ValueError('Unknown SSH agent response: %d' % resptype)
 
-    @asyncio.coroutine
-    def add_smartcard_keys(self, provider, pin=None,
-                           lifetime=None, confirm=False):
+    async def add_smartcard_keys(self, provider, pin=None,
+                                 lifetime=None, confirm=False):
         """Store keys associated with a smart card in the agent
 
-           :param str provider:
+           :param provider:
                The name of the smart card provider
            :param pin: (optional)
                The PIN to use to unlock the smart card
            :param lifetime: (optional)
                The time in seconds after which the keys should be
-               automatically deleted, or ``None`` to store these keys
+               automatically deleted, or `None` to store these keys
                indefinitely (the default).
-           :param bool confirm: (optional)
+           :param confirm: (optional)
                Whether or not to require confirmation for each private
-               key operation which uses these keys, defaulting to ``False``.
-           :type pin: `str` or ``None``
-           :type lifetime: `int` or ``None``
+               key operation which uses these keys, defaulting to `False`.
+           :type provider: `str`
+           :type pin: `str` or `None`
+           :type lifetime: `int` or `None`
+           :type confirm: `bool`
 
            :raises: :exc:`ValueError` if the keys cannot be added
 
@@ -357,9 +381,9 @@ class SSHAgentClient:
         msgtype = SSH_AGENTC_ADD_SMARTCARD_KEY_CONSTRAINED \
                       if constraints else SSH_AGENTC_ADD_SMARTCARD_KEY
 
-        resptype, resp = \
-            yield from self._make_request(msgtype, String(provider),
-                                          String(pin or ''), constraints)
+        resptype, resp = await self._make_request(msgtype, String(provider),
+                                                  String(pin or ''),
+                                                  constraints)
 
         if resptype == SSH_AGENT_SUCCESS:
             resp.check_end()
@@ -368,13 +392,12 @@ class SSHAgentClient:
         else:
             raise ValueError('Unknown SSH agent response: %d' % resptype)
 
-    @asyncio.coroutine
-    def remove_keys(self, keylist):
+    async def remove_keys(self, keylist):
         """Remove a key stored in the agent
 
            :param keylist:
                The list of keys to remove.
-           :type keylist: list of :class:`SSHKeyPair`
+           :type keylist: `list` of :class:`SSHKeyPair`
 
            :raises: :exc:`ValueError` if any keys are not found
 
@@ -382,8 +405,8 @@ class SSHAgentClient:
 
         for keypair in keylist:
             resptype, resp = \
-                yield from self._make_request(SSH_AGENTC_REMOVE_IDENTITY,
-                                              String(keypair.public_data))
+                await self._make_request(SSH_AGENTC_REMOVE_IDENTITY,
+                                         String(keypair.public_data))
 
             if resptype == SSH_AGENT_SUCCESS:
                 resp.check_end()
@@ -392,23 +415,23 @@ class SSHAgentClient:
             else:
                 raise ValueError('Unknown SSH agent response: %d' % resptype)
 
-    @asyncio.coroutine
-    def remove_smartcard_keys(self, provider, pin=None):
+    async def remove_smartcard_keys(self, provider, pin=None):
         """Remove keys associated with a smart card stored in the agent
 
-           :param str provider:
+           :param provider:
                The name of the smart card provider
            :param pin: (optional)
                The PIN to use to unlock the smart card
-           :type pin: `str` or ``None``
+           :type provider: `str`
+           :type pin: `str` or `None`
 
            :raises: :exc:`ValueError` if the keys are not found
 
         """
 
         resptype, resp = \
-            yield from self._make_request(SSH_AGENTC_REMOVE_SMARTCARD_KEY,
-                                          String(provider), String(pin or ''))
+            await self._make_request(SSH_AGENTC_REMOVE_SMARTCARD_KEY,
+                                     String(provider), String(pin or ''))
 
         if resptype == SSH_AGENT_SUCCESS:
             resp.check_end()
@@ -417,8 +440,7 @@ class SSHAgentClient:
         else:
             raise ValueError('Unknown SSH agent response: %d' % resptype)
 
-    @asyncio.coroutine
-    def remove_all(self):
+    async def remove_all(self):
         """Remove all keys stored in the agent
 
            :raises: :exc:`ValueError` if the keys can't be removed
@@ -426,7 +448,7 @@ class SSHAgentClient:
         """
 
         resptype, resp = \
-            yield from self._make_request(SSH_AGENTC_REMOVE_ALL_IDENTITIES)
+            await self._make_request(SSH_AGENTC_REMOVE_ALL_IDENTITIES)
 
         if resptype == SSH_AGENT_SUCCESS:
             resp.check_end()
@@ -435,19 +457,22 @@ class SSHAgentClient:
         else:
             raise ValueError('Unknown SSH agent response: %d' % resptype)
 
-    @asyncio.coroutine
-    def lock(self, passphrase):
+    async def lock(self, passphrase):
         """Lock the agent using the specified passphrase
 
-           :param str passphrase:
+           .. note:: The lock and unlock actions don't appear to be
+                     supported on the Windows 10 OpenSSH agent.
+
+           :param passphrase:
                The passphrase required to later unlock the agent
+           :type passphrase: `str`
 
            :raises: :exc:`ValueError` if the agent can't be locked
 
         """
 
-        resptype, resp = yield from self._make_request(SSH_AGENTC_LOCK,
-                                                       String(passphrase))
+        resptype, resp = await self._make_request(SSH_AGENTC_LOCK,
+                                                  String(passphrase))
 
         if resptype == SSH_AGENT_SUCCESS:
             resp.check_end()
@@ -456,19 +481,22 @@ class SSHAgentClient:
         else:
             raise ValueError('Unknown SSH agent response: %d' % resptype)
 
-    @asyncio.coroutine
-    def unlock(self, passphrase):
+    async def unlock(self, passphrase):
         """Unlock the agent using the specified passphrase
 
-           :param str passphrase:
+           .. note:: The lock and unlock actions don't appear to be
+                     supported on the Windows 10 OpenSSH agent.
+
+           :param passphrase:
                The passphrase to use to unlock the agent
+           :type passphrase: `str`
 
            :raises: :exc:`ValueError` if the agent can't be unlocked
 
         """
 
-        resptype, resp = yield from self._make_request(SSH_AGENTC_UNLOCK,
-                                                       String(passphrase))
+        resptype, resp = await self._make_request(SSH_AGENTC_UNLOCK,
+                                                  String(passphrase))
 
         if resptype == SSH_AGENT_SUCCESS:
             resp.check_end()
@@ -477,16 +505,15 @@ class SSHAgentClient:
         else:
             raise ValueError('Unknown SSH agent response: %d' % resptype)
 
-    @asyncio.coroutine
-    def query_extensions(self):
+    async def query_extensions(self):
         """Return a list of extensions supported by the agent
 
            :returns: A list of strings of supported extension names
 
         """
 
-        resptype, resp = yield from self._make_request(SSH_AGENTC_EXTENSION,
-                                                       String('query'))
+        resptype, resp = await self._make_request(SSH_AGENTC_EXTENSION,
+                                                  String('query'))
 
         if resptype == SSH_AGENT_SUCCESS:
             result = []
@@ -516,57 +543,76 @@ class SSHAgentClient:
 
         """
 
-        self._cleanup()
+        if self._writer:
+            self._writer.close()
+
+    async def wait_closed(self):
+        """Wait for this agent connection to close
+
+           This method is a coroutine which can be called to block until
+           the connection to the agent has finished closing.
+
+        """
+
+        if self._writer:
+            await maybe_wait_closed(self._writer)
+
+            self._reader = None
+            self._writer = None
 
 
-@asyncio.coroutine
-def connect_agent(agent_path=None, *, loop=None):
+class SSHAgentListener:
+    """Listener used to forward agent connections"""
+
+    def __init__(self, tempdir, path, unix_listener):
+        self._tempdir = tempdir
+        self._path = path
+        self._unix_listener = unix_listener
+
+    def get_path(self):
+        """Return the path being listened on"""
+
+        return self._path
+
+    def close(self):
+        """Close the agent listener"""
+
+        self._unix_listener.close()
+        self._tempdir.cleanup()
+
+
+@async_context_manager
+async def connect_agent(agent_path=None):
     """Make a connection to the SSH agent
 
        This function attempts to connect to an ssh-agent process
-       listening on a UNIX domain socket at ``agent_path``. If not
-       provided, it will attempt to get the path from the ``SSH_AUTH_SOCK``
+       listening on a UNIX domain socket at `agent_path`. If not
+       provided, it will attempt to get the path from the `SSH_AUTH_SOCK`
        environment variable.
 
-       If the connection is successful, an ``SSHAgentClient`` object
+       If the connection is successful, an :class:`SSHAgentClient` object
        is returned that has methods on it you can use to query the
        ssh-agent. If no path is specified and the environment variable
-       is not set or the connection to the agent fails, this function
-       returns ``None``.
+       is not set or the connection to the agent fails, an error is
+       raised.
 
        :param agent_path: (optional)
            The path to use to contact the ssh-agent process, or the
            :class:`SSHServerConnection` to forward the agent request
            over.
-       :param loop: (optional)
-           The event loop to use when creating the connection. If not
-           specified, the default event loop is used.
-       :type agent_path: str or :class:`SSHServerConnection`
+       :type agent_path: `str` or :class:`SSHServerConnection`
 
-       :returns: An :class:`SSHAgentClient` or ``None``
+       :returns: An :class:`SSHAgentClient`
+
+       :raises: :exc:`OSError` or :exc:`ChannelOpenError` if the
+                connection to the agent can't be opened
 
     """
 
-    agent = SSHAgentClient(loop, agent_path)
+    if not agent_path:
+        agent_path = os.environ.get('SSH_AUTH_SOCK', None)
 
-    try:
-        yield from agent.connect()
-        return agent
-    except (OSError, ChannelOpenError) as exc:
-        logger.debug('Unable to contact agent: %s', exc)
-        return None
+    agent = SSHAgentClient(agent_path)
+    await agent.connect()
 
-
-@asyncio.coroutine
-def create_agent_listener(conn, loop):
-    """Create a listener for forwarding ssh-agent connections"""
-
-    try:
-        tempdir = tempfile.TemporaryDirectory(prefix='asyncssh-')
-        path = os.path.join(tempdir.name, 'agent')
-        unix_listener = yield from create_unix_forward_listener(
-            conn, loop, conn.create_agent_connection, path)
-
-        return _X11AgentListener(tempdir, path, unix_listener)
-    except OSError:
-        return None
+    return agent
