@@ -1,11 +1,19 @@
-# Copyright (c) 2013-2017 by Ron Frederick <ronf@timeheart.net>.
-# All rights reserved.
+# Copyright (c) 2013-2019 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
-# the terms of the Eclipse Public License v1.0 which accompanies this
+# the terms of the Eclipse Public License v2.0 which accompanies this
 # distribution and is available at:
 #
-#     http://www.eclipse.org/legal/epl-v10.html
+#     http://www.eclipse.org/legal/epl-2.0/
+#
+# This program may also be made available under the following secondary
+# licenses when the conditions for such availability set forth in the
+# Eclipse Public License v2.0 are satisfied:
+#
+#    GNU General Public License, Version 2.0, or any later versions of
+#    that license
+#
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 #
 # Contributors:
 #     Ron Frederick - initial implementation, API, and documentation
@@ -15,14 +23,11 @@
 from .asn1 import ASN1DecodeError, ObjectIdentifier, der_encode, der_decode
 from .crypto import DSAPrivateKey, DSAPublicKey
 from .misc import all_ints
-from .packet import MPInt
+from .packet import MPInt, String
 from .public_key import SSHKey, SSHOpenSSHCertificateV01, KeyExportError
 from .public_key import register_public_key_alg, register_certificate_alg
 from .public_key import register_x509_certificate_alg
 
-# Short variable names are used here, matching names in the spec
-# pylint: disable=invalid-name
-
 
 class _DSAKey(SSHKey):
     """Handler for DSA public key encryption"""
@@ -50,11 +55,9 @@ class _DSAKey(SSHKey):
                      self._key.y, self._key.x))
 
     @classmethod
-    def generate(cls, algorithm):
+    def generate(cls, _algorithm):
         """Generate a new DSA private key"""
 
-        # pylint: disable=unused-argument
-
         return cls(DSAPrivateKey.generate(key_size=1024))
 
     @classmethod
@@ -197,31 +200,24 @@ class _DSAKey(SSHKey):
 
         return MPInt(self._key.x)
 
-    def sign_der(self, data, sig_algorithm):
-        """Compute a DER-encoded signature of the specified data"""
+    def sign_ssh(self, data, sig_algorithm):
+        """Compute an SSH-encoded signature of the specified data"""
 
         # pylint: disable=unused-argument
 
         if not self._key.x:
             raise ValueError('Private key needed for signing')
 
-        return self._key.sign(data)
+        r, s = der_decode(self._key.sign(data))
+        return String(r.to_bytes(20, 'big') + s.to_bytes(20, 'big'))
 
-    def verify_der(self, data, sig_algorithm, sig):
-        """Verify a DER-encoded signature of the specified data"""
+    def verify_ssh(self, data, sig_algorithm, packet):
+        """Verify an SSH-encoded signature of the specified data"""
 
         # pylint: disable=unused-argument
 
-        return self._key.verify(data, sig)
-
-    def sign_ssh(self, data, sig_algorithm):
-        """Compute an SSH-encoded signature of the specified data"""
-
-        r, s = der_decode(self.sign_der(data, sig_algorithm))
-        return r.to_bytes(20, 'big') + s.to_bytes(20, 'big')
-
-    def verify_ssh(self, data, sig_algorithm, sig):
-        """Verify an SSH-encoded signature of the specified data"""
+        sig = packet.get_string()
+        packet.check_end()
 
         if len(sig) != 40:
             return False
@@ -229,7 +225,7 @@ class _DSAKey(SSHKey):
         r = int.from_bytes(sig[:20], 'big')
         s = int.from_bytes(sig[20:], 'big')
 
-        return self.verify_der(data, sig_algorithm, der_encode((r, s)))
+        return self._key.verify(data, der_encode((r, s)))
 
 
 register_public_key_alg(b'ssh-dss', _DSAKey)
