@@ -1,11 +1,19 @@
-# Copyright (c) 2013-2017 by Ron Frederick <ronf@timeheart.net>.
-# All rights reserved.
+# Copyright (c) 2013-2019 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
-# the terms of the Eclipse Public License v1.0 which accompanies this
+# the terms of the Eclipse Public License v2.0 which accompanies this
 # distribution and is available at:
 #
-#     http://www.eclipse.org/legal/epl-v10.html
+#     http://www.eclipse.org/legal/epl-2.0/
+#
+# This program may also be made available under the following secondary
+# licenses when the conditions for such availability set forth in the
+# Eclipse Public License v2.0 are satisfied:
+#
+#    GNU General Public License, Version 2.0, or any later versions of
+#    that license
+#
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 #
 # Contributors:
 #     Ron Frederick - initial implementation, API, and documentation
@@ -25,9 +33,6 @@ from .public_key import register_x509_certificate_alg
 # OID for EC prime fields
 PRIME_FIELD = ObjectIdentifier('1.2.840.10045.1.1')
 
-# Short variable names are used here, matching names in the spec
-# pylint: disable=invalid-name
-
 _alg_oids = {}
 _alg_oid_map = {}
 
@@ -143,10 +148,9 @@ class _ECKey(SSHKey):
             return None
 
     @classmethod
-    def decode_pkcs1_public(cls, key_data):
+    def decode_pkcs1_public(cls, _key_data):
         """Decode a PKCS#1 format EC public key"""
 
-        # pylint: disable=unused-argument
         raise KeyImportError('PKCS#1 not supported for EC public keys')
 
     @classmethod
@@ -261,43 +265,37 @@ class _ECKey(SSHKey):
 
         return MPInt(self._key.d)
 
-    def sign_der(self, data, sig_algorithm):
-        """Compute a DER-encoded signature of the specified data"""
+    def sign_ssh(self, data, sig_algorithm):
+        """Compute an SSH-encoded signature of the specified data"""
 
         # pylint: disable=unused-argument
 
         if not self._key.private_value:
             raise ValueError('Private key needed for signing')
 
-        return self._key.sign(data)
+        r, s = der_decode(self._key.sign(data))
+        return String(MPInt(r) + MPInt(s))
 
-    def verify_der(self, data, sig_algorithm, sig):
-        """Verify a DER-encoded signature of the specified data"""
+    def verify_ssh(self, data, sig_algorithm, packet):
+        """Verify an SSH-encoded signature of the specified data"""
 
         # pylint: disable=unused-argument
 
-        return self._key.verify(data, sig)
-
-    def sign_ssh(self, data, sig_algorithm):
-        """Compute an SSH-encoded signature of the specified data"""
-
-        r, s = der_decode(self.sign_der(data, sig_algorithm))
-        return MPInt(r) + MPInt(s)
-
-    def verify_ssh(self, data, sig_algorithm, sig):
-        """Verify an SSH-encoded signature of the specified data"""
+        sig = packet.get_string()
+        packet.check_end()
 
         packet = SSHPacket(sig)
         r = packet.get_mpint()
         s = packet.get_mpint()
         packet.check_end()
 
-        return self.verify_der(data, sig_algorithm, der_encode((r, s)))
+        return self._key.verify(data, der_encode((r, s)))
 
 
 for _curve_id, _oid in ((b'nistp521', '1.3.132.0.35'),
                         (b'nistp384', '1.3.132.0.34'),
-                        (b'nistp256', '1.2.840.10045.3.1.7')):
+                        (b'nistp256', '1.2.840.10045.3.1.7'),
+                        (b'1.3.132.0.10', '1.3.132.0.10')):
     _algorithm = b'ecdsa-sha2-' + _curve_id
     _cert_algorithm = _algorithm + b'-cert-v01@openssh.com'
     _x509_algorithm = b'x509v3-' + _algorithm
