@@ -1,11 +1,19 @@
-# Copyright (c) 2015-2017 by Ron Frederick <ronf@timeheart.net>.
-# All rights reserved.
+# Copyright (c) 2015-2019 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
-# the terms of the Eclipse Public License v1.0 which accompanies this
+# the terms of the Eclipse Public License v2.0 which accompanies this
 # distribution and is available at:
 #
-#     http://www.eclipse.org/legal/epl-v10.html
+#     http://www.eclipse.org/legal/epl-2.0/
+#
+# This program may also be made available under the following secondary
+# licenses when the conditions for such availability set forth in the
+# Eclipse Public License v2.0 are satisfied:
+#
+#    GNU General Public License, Version 2.0, or any later versions of
+#    that license
+#
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 #
 # Contributors:
 #     Ron Frederick - initial implementation, API, and documentation
@@ -79,11 +87,9 @@ class _HashedHost:
         hosthash = hmac.new(self._salt, value.encode(), sha1).digest()
         return hosthash == self._hosthash
 
-    def matches(self, host, addr, ip):
+    def matches(self, host, addr, _ip):
         """Return whether a host or address matches this host hash"""
 
-        # pylint: disable=unused-argument
-
         return (host and self._match(host)) or (addr and self._match(addr))
 
 
@@ -161,7 +167,13 @@ class SSHKnownHosts:
     def _match(self, host, addr, port=None):
         """Find host keys matching specified host, address, and port"""
 
-        ip = ip_address(addr) if addr else None
+        if addr:
+            ip = ip_address(addr)
+        else:
+            try:
+                ip = ip_address(host)
+            except ValueError:
+                ip = None
 
         if port:
             host = '[{}]:{}'.format(host, port) if host else None
