@@ -1,11 +1,19 @@
-# Copyright (c) 2013-2016 by Ron Frederick <ronf@timeheart.net>.
-# All rights reserved.
+# Copyright (c) 2013-2019 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
-# the terms of the Eclipse Public License v1.0 which accompanies this
+# the terms of the Eclipse Public License v2.0 which accompanies this
 # distribution and is available at:
 #
-#     http://www.eclipse.org/legal/epl-v10.html
+#     http://www.eclipse.org/legal/epl-2.0/
+#
+# This program may also be made available under the following secondary
+# licenses when the conditions for such availability set forth in the
+# Eclipse Public License v2.0 are satisfied:
+#
+#    GNU General Public License, Version 2.0, or any later versions of
+#    that license
+#
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 #
 # Contributors:
 #     Ron Frederick - initial implementation, API, and documentation
@@ -17,11 +25,19 @@ import errno
 import socket
 
 from .forward import SSHLocalPortForwarder, SSHLocalPathForwarder
+from .socks import SSHSOCKSForwarder
 
 
-class SSHListener(asyncio.AbstractServer):
+class SSHListener:
     """SSH listener for inbound connections"""
 
+    async def __aenter__(self):
+        return self
+
+    async def __aexit__(self, *exc_info):
+        self.close()
+        await self.wait_closed()
+
     def get_port(self):
         """Return the port number being listened on
 
@@ -49,8 +65,7 @@ class SSHListener(asyncio.AbstractServer):
 
         raise NotImplementedError
 
-    @asyncio.coroutine
-    def wait_closed(self):
+    async def wait_closed(self):
         """Wait for the listener to close
 
            This method is a coroutine which waits for the associated
@@ -64,17 +79,17 @@ class SSHListener(asyncio.AbstractServer):
 class SSHClientListener(SSHListener):
     """Client listener used to accept inbound forwarded connections"""
 
-    def __init__(self, conn, loop, session_factory,
-                 encoding, window, max_pktsize):
+    def __init__(self, conn, session_factory,
+                 encoding, errors, window, max_pktsize):
         self._conn = conn
         self._session_factory = session_factory
         self._encoding = encoding
+        self._errors = errors
         self._window = window
         self._max_pktsize = max_pktsize
-        self._close_event = asyncio.Event(loop=loop)
+        self._close_event = asyncio.Event()
 
-    @asyncio.coroutine
-    def _close(self):
+    async def _close(self):
         """Close this listener"""
 
         self._close_event.set()
@@ -86,39 +101,37 @@ class SSHClientListener(SSHListener):
         if self._conn:
             self._conn.create_task(self._close())
 
-    @asyncio.coroutine
-    def wait_closed(self):
+    async def wait_closed(self):
         """Wait for this listener to finish closing"""
 
-        yield from self._close_event.wait()
+        await self._close_event.wait()
 
 
 class SSHTCPClientListener(SSHClientListener):
     """Client listener used to accept inbound forwarded TCP connections"""
 
-    def __init__(self, conn, loop, session_factory, listen_host, listen_port,
-                 encoding, window, max_pktsize):
-        super().__init__(conn, loop, session_factory,
-                         encoding, window, max_pktsize)
+    def __init__(self, conn, session_factory, listen_host, listen_port,
+                 encoding, errors, window, max_pktsize):
+        super().__init__(conn, session_factory, encoding,
+                         errors, window, max_pktsize)
 
         self._listen_host = listen_host
         self._listen_port = listen_port
 
-    @asyncio.coroutine
-    def _close(self):
+    async def _close(self):
         """Close this listener"""
 
         if self._conn: # pragma: no branch
-            yield from self._conn.close_client_tcp_listener(self._listen_host,
-                                                            self._listen_port)
+            await self._conn.close_client_tcp_listener(self._listen_host,
+                                                       self._listen_port)
 
-        yield from super()._close()
+        await super()._close()
 
     def process_connection(self, orig_host, orig_port):
         """Process a forwarded TCP connection"""
 
-        chan = self._conn.create_tcp_channel(self._encoding, self._window,
-                                             self._max_pktsize)
+        chan = self._conn.create_tcp_channel(self._encoding, self._errors,
+                                             self._window, self._max_pktsize)
 
         chan.set_inbound_peer_names(self._listen_host, self._listen_port,
                                     orig_host, orig_port)
@@ -134,27 +147,26 @@ class SSHTCPClientListener(SSHClientListener):
 class SSHUNIXClientListener(SSHClientListener):
     """Client listener used to accept inbound forwarded UNIX connections"""
 
-    def __init__(self, conn, loop, session_factory, listen_path,
-                 encoding, window, max_pktsize):
-        super().__init__(conn, loop, session_factory,
-                         encoding, window, max_pktsize)
+    def __init__(self, conn, session_factory, listen_path,
+                 encoding, errors, window, max_pktsize):
+        super().__init__(conn, session_factory, encoding,
+                         errors, window, max_pktsize)
 
         self._listen_path = listen_path
 
-    @asyncio.coroutine
-    def _close(self):
+    async def _close(self):
         """Close this listener"""
 
         if self._conn: # pragma: no branch
-            yield from self._conn.close_client_unix_listener(self._listen_path)
+            await self._conn.close_client_unix_listener(self._listen_path)
 
-        yield from super()._close()
+        await super()._close()
 
     def process_connection(self):
         """Process a forwarded UNIX connection"""
 
-        chan = self._conn.create_unix_channel(self._encoding, self._window,
-                                              self._max_pktsize)
+        chan = self._conn.create_unix_channel(self._encoding, self._errors,
+                                              self._window, self._max_pktsize)
 
         chan.set_inbound_peer_names(self._listen_path)
 
@@ -179,32 +191,26 @@ class SSHForwardListener(SSHListener):
         for server in self._servers:
             server.close()
 
-    @asyncio.coroutine
-    def wait_closed(self):
+    async def wait_closed(self):
         """Wait for this listener to finish closing"""
 
         for server in self._servers:
-            yield from server.wait_closed()
+            await server.wait_closed()
 
         self._servers = []
 
 
-@asyncio.coroutine
-def create_tcp_forward_listener(conn, loop, coro, listen_host, listen_port):
-    """Create a listener to forward traffic from local ports over SSH"""
-
-    def protocol_factory():
-        """Start a port forwarder for each new local connection"""
-
-        return SSHLocalPortForwarder(conn, coro)
+async def create_tcp_local_listener(conn, loop, protocol_factory,
+                                    listen_host, listen_port):
+    """Create a listener to forward traffic from a local TCP port over SSH"""
 
     if listen_host == '':
         listen_host = None
 
-    addrinfo = yield from loop.getaddrinfo(listen_host, listen_port,
-                                           family=socket.AF_UNSPEC,
-                                           type=socket.SOCK_STREAM,
-                                           flags=socket.AI_PASSIVE)
+    addrinfo = await loop.getaddrinfo(listen_host, listen_port,
+                                      family=socket.AF_UNSPEC,
+                                      type=socket.SOCK_STREAM,
+                                      flags=socket.AI_PASSIVE)
 
     if not addrinfo: # pragma: no cover
         raise OSError('getaddrinfo() returned empty list')
@@ -240,6 +246,7 @@ def create_tcp_forward_listener(conn, loop, coro, listen_host, listen_port):
                 exc.errno = errno.EOVERFLOW
                 exc.strerror = str(exc)
 
+            # pylint: disable=no-member
             raise OSError(exc.errno, 'error while attempting to bind on '
                           'address %r: %s' % (sa, exc.strerror)) from None
 
@@ -247,14 +254,26 @@ def create_tcp_forward_listener(conn, loop, coro, listen_host, listen_port):
             listen_port = sock.getsockname()[1]
             conn.logger.debug1('Assigning dynamic port %d', listen_port)
 
-        server = yield from loop.create_server(protocol_factory, sock=sock)
+        server = await loop.create_server(protocol_factory, sock=sock)
         servers.append(server)
 
     return SSHForwardListener(servers, listen_port)
 
 
-@asyncio.coroutine
-def create_unix_forward_listener(conn, loop, coro, listen_path):
+async def create_tcp_forward_listener(conn, loop, coro,
+                                      listen_host, listen_port):
+    """Create a listener to forward traffic from a local TCP port over SSH"""
+
+    def protocol_factory():
+        """Start a port forwarder for each new local connection"""
+
+        return SSHLocalPortForwarder(conn, coro)
+
+    return await create_tcp_local_listener(conn, loop, protocol_factory,
+                                           listen_host, listen_port)
+
+
+async def create_unix_forward_listener(conn, loop, coro, listen_path):
     """Create a listener to forward a local UNIX domain socket over SSH"""
 
     def protocol_factory():
@@ -262,6 +281,18 @@ def create_unix_forward_listener(conn, loop, coro, listen_path):
 
         return SSHLocalPathForwarder(conn, coro)
 
-    server = yield from loop.create_unix_server(protocol_factory, listen_path)
+    server = await loop.create_unix_server(protocol_factory, listen_path)
 
     return SSHForwardListener([server])
+
+
+async def create_socks_listener(conn, loop, coro, listen_host, listen_port):
+    """Create a SOCKS listener to forward traffic over SSH"""
+
+    def protocol_factory():
+        """Start a port forwarder for each new SOCKS connection"""
+
+        return SSHSOCKSForwarder(conn, coro)
+
+    return await create_tcp_local_listener(conn, loop, protocol_factory,
+                                           listen_host, listen_port)
