@@ -1,11 +1,19 @@
-# Copyright (c) 2016-2017 by Ron Frederick <ronf@timeheart.net>.else:
-# All rights reserved.
+# Copyright (c) 2016-2019 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
-# the terms of the Eclipse Public License v1.0 which accompanies this
+# the terms of the Eclipse Public License v2.0 which accompanies this
 # distribution and is available at:
 #
-#     http://www.eclipse.org/legal/epl-v10.html
+#     http://www.eclipse.org/legal/epl-2.0/
+#
+# This program may also be made available under the following secondary
+# licenses when the conditions for such availability set forth in the
+# Eclipse Public License v2.0 are satisfied:
+#
+#    GNU General Public License, Version 2.0, or any later versions of
+#    that license
+#
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 #
 # Contributors:
 #     Ron Frederick - initial implementation, API, and documentation
@@ -14,14 +22,16 @@
 
 import asyncio
 from asyncio.subprocess import DEVNULL, PIPE, STDOUT
+import codecs
 from collections import OrderedDict
 import io
 import os
+from pathlib import PurePath
 import socket
 import stat
 
-from .constants import DEFAULT_LANG, DISC_PROTOCOL_ERROR, EXTENDED_DATA_STDERR
-from .misc import DisconnectError, Error, Record
+from .constants import DEFAULT_LANG, EXTENDED_DATA_STDERR
+from .misc import Error, ProtocolError, Record
 from .stream import SSHClientStreamSession, SSHServerStreamSession
 from .stream import SSHReader, SSHWriter
 
@@ -38,42 +48,27 @@ def _is_regular_file(file):
 class _UnicodeReader:
     """Handle buffering partial Unicode data"""
 
-    def __init__(self, encoding, textmode=False):
-        self._encoding = encoding
-        self._textmode = textmode
-        self._partial = b''
+    def __init__(self, encoding, errors, textmode=False):
+        if encoding and not textmode:
+            self._decoder = codecs.getincrementaldecoder(encoding)(errors)
+        else:
+            self._decoder = None
 
-    def decode(self, data):
+    def decode(self, data, final=False):
         """Decode Unicode bytes when reading from binary sources"""
 
-        if self._encoding and not self._textmode:
-            data = self._partial + data
-            self._partial = b''
-
+        if self._decoder:
             try:
-                data = data.decode(self._encoding)
-            except UnicodeDecodeError as exc:
-                if exc.start > 0:
-                    # Avoid pylint false positive
-                    # pylint: disable=invalid-slice-index
-                    self._partial = data[exc.start:]
-                    data = data[:exc.start].decode(self._encoding)
-                elif exc.reason == 'unexpected end of data':
-                    self._partial = data
-                    data = ''
-                else:
-                    self.close()
-                    raise DisconnectError(DISC_PROTOCOL_ERROR,
-                                          'Unicode decode error')
+                data = self._decoder.decode(data, final)
+            except UnicodeDecodeError as unicode_exc:
+                raise ProtocolError(str(unicode_exc)) from None
 
         return data
 
     def check_partial(self):
         """Check if there's partial Unicode data left at EOF"""
 
-        if self._partial:
-            self.close()
-            raise DisconnectError(DISC_PROTOCOL_ERROR, 'Unicode decode error')
+        self.decode(b'', True)
 
     def close(self):
         """Perform necessary cleanup on error (provided by derived classes)"""
@@ -82,15 +77,17 @@ class _UnicodeReader:
 class _UnicodeWriter:
     """Handle encoding Unicode data before writing it"""
 
-    def __init__(self, encoding, textmode=False):
-        self._encoding = encoding
-        self._textmode = textmode
+    def __init__(self, encoding, errors, textmode=False):
+        if encoding and not textmode:
+            self._encoder = codecs.getincrementalencoder(encoding)(errors)
+        else:
+            self._encoder = None
 
     def encode(self, data):
         """Encode Unicode bytes when writing to binary targets"""
 
-        if self._encoding and not self._textmode:
-            data = data.encode(self._encoding)
+        if self._encoder:
+            data = self._encoder.encode(data)
 
         return data
 
@@ -98,8 +95,8 @@ class _UnicodeWriter:
 class _FileReader(_UnicodeReader):
     """Forward data from a file"""
 
-    def __init__(self, process, file, bufsize, datatype, encoding):
-        super().__init__(encoding, hasattr(file, 'encoding'))
+    def __init__(self, process, file, bufsize, datatype, encoding, errors):
+        super().__init__(encoding, errors, hasattr(file, 'encoding'))
 
         self._process = process
         self._file = file
@@ -137,11 +134,43 @@ class _FileReader(_UnicodeReader):
         self._file.close()
 
 
+class _AsyncFileReader(_FileReader):
+    """Forward data from an aiofile"""
+
+    def __init__(self, process, file, bufsize, datatype, encoding, errors):
+        super().__init__(process, file, bufsize, datatype, encoding, errors)
+
+        self._conn = process.channel.get_connection()
+
+    async def _feed(self):
+        """Feed file data"""
+
+        while not self._paused:
+            data = await self._file.read(self._bufsize)
+
+            if data:
+                self._process.feed_data(self.decode(data), self._datatype)
+            else:
+                self.check_partial()
+                self._process.feed_eof(self._datatype)
+                break
+
+    def feed(self):
+        """Start feeding file data"""
+
+        self._conn.create_task(self._feed())
+
+    def close(self):
+        """Stop forwarding data from the file"""
+
+        self._conn.create_task(self._file.close())
+
+
 class _FileWriter(_UnicodeWriter):
     """Forward data to a file"""
 
-    def __init__(self, file, encoding):
-        super().__init__(encoding, hasattr(file, 'encoding'))
+    def __init__(self, file, encoding, errors):
+        super().__init__(encoding, errors, hasattr(file, 'encoding'))
 
         self._file = file
 
@@ -161,11 +190,30 @@ class _FileWriter(_UnicodeWriter):
         self._file.close()
 
 
+class _AsyncFileWriter(_FileWriter):
+    """Forward data to an aiofile"""
+
+    def __init__(self, process, file, encoding, errors):
+        super().__init__(file, encoding, errors)
+
+        self._conn = process.channel.get_connection()
+
+    def write(self, data):
+        """Write data to the file"""
+
+        self._conn.create_task(self._file.write(self.encode(data)))
+
+    def close(self):
+        """Stop forwarding data to the file"""
+
+        self._conn.create_task(self._file.close())
+
+
 class _PipeReader(_UnicodeReader, asyncio.Protocol):
     """Forward data from a pipe"""
 
-    def __init__(self, process, datatype, encoding):
-        super().__init__(encoding)
+    def __init__(self, process, datatype, encoding, errors):
+        super().__init__(encoding, errors)
 
         self._process = process
         self._datatype = datatype
@@ -206,8 +254,8 @@ class _PipeReader(_UnicodeReader, asyncio.Protocol):
 class _PipeWriter(_UnicodeWriter, asyncio.BaseProtocol):
     """Forward data to a pipe"""
 
-    def __init__(self, process, datatype, encoding):
-        super().__init__(encoding)
+    def __init__(self, process, datatype, encoding, errors):
+        super().__init__(encoding, errors)
 
         self._process = process
         self._datatype = datatype
@@ -290,6 +338,74 @@ class _ProcessWriter:
         self._process.clear_reader(self._datatype)
 
 
+class _StreamReader(_UnicodeReader):
+    """Forward data from an asyncio stream"""
+
+    def __init__(self, process, reader, bufsize, datatype, encoding, errors):
+        super().__init__(encoding, errors)
+
+        self._process = process
+        self._conn = process.channel.get_connection()
+        self._reader = reader
+        self._bufsize = bufsize
+        self._datatype = datatype
+        self._paused = False
+
+    async def _feed(self):
+        """Feed stream data"""
+
+        while not self._paused:
+            data = await self._reader.read(self._bufsize)
+
+            if data:
+                self._process.feed_data(self.decode(data), self._datatype)
+            else:
+                self.check_partial()
+                self._process.feed_eof(self._datatype)
+                break
+
+    def feed(self):
+        """Start feeding stream data"""
+
+        self._conn.create_task(self._feed())
+
+    def pause_reading(self):
+        """Pause reading from the stream"""
+
+        self._paused = True
+
+    def resume_reading(self):
+        """Resume reading from the stream"""
+
+        self._paused = False
+        self.feed()
+
+    def close(self):
+        """Ignore close -- the caller must clean up the associated transport"""
+
+
+class _StreamWriter(_UnicodeWriter):
+    """Forward data to an asyncio stream"""
+
+    def __init__(self, writer, encoding, errors):
+        super().__init__(encoding, errors)
+
+        self._writer = writer
+
+    def write(self, data):
+        """Write data to the stream"""
+
+        self._writer.write(self.encode(data))
+
+    def write_eof(self):
+        """Write EOF to the stream"""
+
+        self._writer.write_eof()
+
+    def close(self):
+        """Ignore close -- the caller must clean up the associated transport"""
+
+
 class _DevNullWriter:
     """Discard data"""
 
@@ -338,7 +454,7 @@ class ProcessError(Error):
                       process to execute (if any)
          subsystem    The subsystem the client requested the  `str` or `None`
                       process to open (if any)
-         exit_status  The exit status returned, or -1 if an   `int`
+         exit_status  The exit status returned, or -1 if an   `int` or `None`
                       exit signal is sent
          exit_signal  The exit signal sent (if any) in the    `tuple` or `None`
                       form of a tuple containing the signal
@@ -346,6 +462,9 @@ class ProcessError(Error):
                       occurred, a message associated with the
                       signal, and the language the message
                       was in
+         returncode   The exit status returned, or negative   `int` or `None`
+                      of the signal number when an exit
+                      signal is sent
          stdout       The output sent by the process to       `str` or `bytes`
                       stdout (if not redirected)
          stderr       The output sent by the process to       `str` or `bytes`
@@ -355,12 +474,14 @@ class ProcessError(Error):
     """
 
     def __init__(self, env, command, subsystem, exit_status,
-                 exit_signal, stdout, stderr):
+                 exit_signal, returncode, stdout, stderr,
+                 reason='', lang=DEFAULT_LANG):
         self.env = env
         self.command = command
         self.subsystem = subsystem
         self.exit_status = exit_status
         self.exit_signal = exit_signal
+        self.returncode = returncode
         self.stdout = stdout
         self.stderr = stderr
 
@@ -369,12 +490,28 @@ class ProcessError(Error):
             reason = 'Process exited with signal %s%s%s' % \
                 (signal, ': ' + msg if msg else '',
                  ' (core dumped)' if core_dumped else '')
-        else:
+        elif exit_status:
             reason = 'Process exited with non-zero exit status %s' % \
                 exit_status
-            lang = DEFAULT_LANG
 
-        super().__init__('Process', exit_status, reason, lang)
+        super().__init__(exit_status, reason, lang)
+
+
+# pylint: disable=redefined-builtin
+class TimeoutError(ProcessError, asyncio.TimeoutError):
+    """SSH Process timeout error
+
+       This exception is raised when a timeout occurs when calling the
+       :meth:`wait <SSHClientProcess.wait>` method on :class:`SSHClientProcess`
+       or the :meth:`run <SSHClientConnection.run>` method on
+       :class:`SSHClientConnection`. It is a subclass of :class:`ProcessError`
+       and contains all of the fields documented there, including any output
+       received on stdout and stderr prior to when the timeout occurred. It
+       is also a subclass of :class:`asyncio.TimeoutError`, for code that
+       might be expecting that.
+
+    """
+# pylint: enable=redefined-builtin
 
 
 class SSHCompletedProcess(Record):
@@ -401,6 +538,9 @@ class SSHCompletedProcess(Record):
                       occurred, a message associated with the
                       signal, and the language the message
                       was in
+         returncode   The exit status returned, or negative   `int`
+                      of the signal number when an exit
+                      signal is sent
          stdout       The output sent by the process to       `str` or `bytes`
                       stdout (if not redirected)
          stderr       The output sent by the process to       `str` or `bytes`
@@ -411,8 +551,8 @@ class SSHCompletedProcess(Record):
 
     __slots__ = OrderedDict((('env', None), ('command', None),
                              ('subsystem', None), ('exit_status', None),
-                             ('exit_signal', None), ('stdout', None),
-                             ('stderr', None)))
+                             ('exit_signal', None), ('returncode', None),
+                             ('stdout', None), ('stderr', None)))
 
 
 class SSHProcess:
@@ -433,28 +573,16 @@ class SSHProcess:
         self._stdout = None
         self._stderr = None
 
-    def __enter__(self):
-        """Allow SSHProcess to be used as a context manager"""
-
-        return self
-
-    def __exit__(self, *exc_info):
-        """Automatically close the channel when exiting the context"""
-
-        self.close()
-
-    @asyncio.coroutine
-    def __aenter__(self):
+    async def __aenter__(self):
         """Allow SSHProcess to be used as an async context manager"""
 
         return self
 
-    @asyncio.coroutine
-    def __aexit__(self, *exc_info):
+    async def __aexit__(self, *exc_info):
         """Wait for a full channel close when exiting the async context"""
 
         self.close()
-        yield from self._chan.wait_closed()
+        await self._chan.wait_closed()
 
     @property
     def channel(self):
@@ -514,14 +642,25 @@ class SSHProcess:
 
         return self._chan.get_subsystem()
 
-    @asyncio.coroutine
-    def _create_reader(self, source, bufsize, send_eof, datatype=None):
+    def get_extra_info(self, name, default=None):
+        """Return additional information about this process
+
+           This method returns extra information about the channel
+           associated with this process. See :meth:`get_extra_info()
+           <SSHClientChannel.get_extra_info>` on :class:`SSHClientChannel`
+           for additional information.
+
+        """
+
+        return self._chan.get_extra_info(name, default)
+
+    async def _create_reader(self, source, bufsize, send_eof, datatype=None):
         """Create a reader to forward data to the SSH channel"""
 
         def pipe_factory():
             """Return a pipe read handler"""
 
-            return _PipeReader(self, datatype, self._encoding)
+            return _PipeReader(self, datatype, self._encoding, self._errors)
 
         if source == PIPE:
             reader = None
@@ -533,9 +672,14 @@ class SSHProcess:
             writer = _ProcessWriter(self, datatype)
             reader_process.set_writer(writer, reader_datatype)
             reader = _ProcessReader(reader_process, reader_datatype)
+        elif isinstance(source, asyncio.StreamReader):
+            reader = _StreamReader(self, source, bufsize, datatype,
+                                   self._encoding, self._errors)
         else:
             if isinstance(source, str):
                 file = open(source, 'rb', buffering=bufsize)
+            elif isinstance(source, PurePath):
+                file = open(str(source), 'rb', buffering=bufsize)
             elif isinstance(source, int):
                 file = os.fdopen(source, 'rb', buffering=bufsize)
             elif isinstance(source, socket.socket):
@@ -543,32 +687,34 @@ class SSHProcess:
             else:
                 file = source
 
-            if _is_regular_file(file):
-                reader = _FileReader(self, file, bufsize,
-                                     datatype, self._encoding)
+            if hasattr(file, 'read') and asyncio.iscoroutinefunction(file.read):
+                reader = _AsyncFileReader(self, file, bufsize, datatype,
+                                          self._encoding, self._errors)
+            elif _is_regular_file(file):
+                reader = _FileReader(self, file, bufsize, datatype,
+                                     self._encoding, self._errors)
             else:
                 if hasattr(source, 'buffer'):
                     # If file was opened in text mode, remove that wrapper
                     file = source.buffer
 
                 _, reader = \
-                    yield from self._loop.connect_read_pipe(pipe_factory, file)
+                    await self._loop.connect_read_pipe(pipe_factory, file)
 
         self.set_reader(reader, send_eof, datatype)
 
-        if isinstance(reader, _FileReader):
+        if isinstance(reader, (_FileReader, _StreamReader)):
             reader.feed()
         elif isinstance(reader, _ProcessReader):
             reader_process.feed_recv_buf(reader_datatype, writer)
 
-    @asyncio.coroutine
-    def _create_writer(self, target, bufsize, send_eof, datatype=None):
+    async def _create_writer(self, target, bufsize, send_eof, datatype=None):
         """Create a writer to forward data from the SSH channel"""
 
         def pipe_factory():
             """Return a pipe write handler"""
 
-            return _PipeWriter(self, datatype, self._encoding)
+            return _PipeWriter(self, datatype, self._encoding, self._errors)
 
         if target == DEVNULL:
             writer = _DevNullWriter()
@@ -581,9 +727,13 @@ class SSHProcess:
             reader = _ProcessReader(self, datatype)
             writer_process.set_reader(reader, send_eof, writer_datatype)
             writer = _ProcessWriter(writer_process, writer_datatype)
+        elif isinstance(target, asyncio.StreamWriter):
+            writer = _StreamWriter(target, self._encoding, self._errors)
         else:
             if isinstance(target, str):
                 file = open(target, 'wb', buffering=bufsize)
+            elif isinstance(target, PurePath):
+                file = open(str(target), 'wb', buffering=bufsize)
             elif isinstance(target, int):
                 file = os.fdopen(target, 'wb', buffering=bufsize)
             elif isinstance(target, socket.socket):
@@ -591,16 +741,19 @@ class SSHProcess:
             else:
                 file = target
 
-            if _is_regular_file(file):
-                writer = _FileWriter(file, self._encoding)
+            if hasattr(file, 'write') and \
+                    asyncio.iscoroutinefunction(file.write):
+                writer = _AsyncFileWriter(self, file, self._encoding,
+                                          self._errors)
+            elif _is_regular_file(file):
+                writer = _FileWriter(file, self._encoding, self._errors)
             else:
                 if hasattr(target, 'buffer'):
                     # If file was opened in text mode, remove that wrapper
                     file = target.buffer
 
-                _, writer = \
-                    yield from self._loop.connect_write_pipe(pipe_factory,
-                                                             file)
+                _, writer = await self._loop.connect_write_pipe(pipe_factory,
+                                                                file)
 
         self.set_writer(writer, datatype)
 
@@ -755,11 +908,15 @@ class SSHProcess:
 
         self._chan.close()
 
-    @asyncio.coroutine
-    def wait_closed(self):
+    def is_closing(self):
+        """Return if the channel is closing or is closed"""
+
+        return self._chan.is_closing()
+
+    async def wait_closed(self):
         """Wait for the process to finish shutting down"""
 
-        yield from self._chan.wait_closed()
+        await self._chan.wait_closed()
 
 
 class SSHClientProcess(SSHProcess, SSHClientStreamSession):
@@ -801,6 +958,12 @@ class SSHClientProcess(SSHProcess, SSHClientStreamSession):
 
         return self._chan.get_exit_signal()
 
+    @property
+    def returncode(self):
+        """The exit status or negative exit signal number for the process"""
+
+        return self._chan.get_returncode()
+
     @property
     def stdin(self):
         """The :class:`SSHWriter` to use to write to stdin of the process"""
@@ -819,9 +982,8 @@ class SSHClientProcess(SSHProcess, SSHClientStreamSession):
 
         return self._stderr
 
-    @asyncio.coroutine
-    def redirect(self, stdin=None, stdout=None, stderr=None,
-                 bufsize=io.DEFAULT_BUFFER_SIZE, send_eof=True):
+    async def redirect(self, stdin=None, stdout=None, stderr=None,
+                       bufsize=io.DEFAULT_BUFFER_SIZE, send_eof=True):
         """Perform I/O redirection for the process
 
            This method redirects data going to or from any or all of
@@ -831,20 +993,24 @@ class SSHClientProcess(SSHProcess, SSHClientStreamSession):
            The `stdin` argument can be any of the following:
 
                * An :class:`SSHReader` object
+               * An :class:`asyncio.StreamReader` object
                * A file object open for read
                * An `int` file descriptor open for read
                * A connected socket object
-               * A string containing the name of a file or device to open
+               * A string or :class:`PurePath <pathlib.PurePath>` containing
+                 the name of a file or device to open
                * `DEVNULL` to provide no input to standard input
                * `PIPE` to interactively write standard input
 
            The `stdout` and `stderr` arguments can be any of the following:
 
                * An :class:`SSHWriter` object
+               * An :class:`asyncio.StreamWriter` object
                * A file object open for write
                * An `int` file descriptor open for write
                * A connected socket object
-               * A string containing the name of a file or device to open
+               * A string or :class:`PurePath <pathlib.PurePath>` containing
+                 the name of a file or device to open
                * `DEVNULL` to discard standard error output
                * `PIPE` to interactively read standard error output
 
@@ -857,6 +1023,10 @@ class SSHClientProcess(SSHProcess, SSHClientStreamSession):
            The default value of `None` means to not change redirection
            for that stream.
 
+           .. note:: When passing in asyncio streams, it is the responsibility
+                     of the caller to close the associated transport when it
+                     is no longer needed.
+
            :param stdin:
                Source of data to feed to standard input
            :param stdout:
@@ -875,35 +1045,32 @@ class SSHClientProcess(SSHProcess, SSHClientStreamSession):
         """
 
         if stdin:
-            yield from self._create_reader(stdin, bufsize, send_eof)
+            await self._create_reader(stdin, bufsize, send_eof)
 
         if stdout:
-            yield from self._create_writer(stdout, bufsize, send_eof)
+            await self._create_writer(stdout, bufsize, send_eof)
 
         if stderr:
-            yield from self._create_writer(stderr, bufsize, send_eof,
-                                           EXTENDED_DATA_STDERR)
+            await self._create_writer(stderr, bufsize, send_eof,
+                                      EXTENDED_DATA_STDERR)
 
-    @asyncio.coroutine
-    def redirect_stdin(self, source, bufsize=io.DEFAULT_BUFFER_SIZE,
-                       send_eof=True):
+    async def redirect_stdin(self, source, bufsize=io.DEFAULT_BUFFER_SIZE,
+                             send_eof=True):
         """Redirect standard input of the process"""
 
-        yield from self.redirect(source, None, None, bufsize, send_eof)
+        await self.redirect(source, None, None, bufsize, send_eof)
 
-    @asyncio.coroutine
-    def redirect_stdout(self, target, bufsize=io.DEFAULT_BUFFER_SIZE,
-                        send_eof=True):
+    async def redirect_stdout(self, target, bufsize=io.DEFAULT_BUFFER_SIZE,
+                              send_eof=True):
         """Redirect standard output of the process"""
 
-        yield from self.redirect(None, target, None, bufsize, send_eof)
+        await self.redirect(None, target, None, bufsize, send_eof)
 
-    @asyncio.coroutine
-    def redirect_stderr(self, target, bufsize=io.DEFAULT_BUFFER_SIZE,
-                        send_eof=True):
+    async def redirect_stderr(self, target, bufsize=io.DEFAULT_BUFFER_SIZE,
+                              send_eof=True):
         """Redirect standard error of the process"""
 
-        yield from self.redirect(None, None, target, bufsize, send_eof)
+        await self.redirect(None, None, target, bufsize, send_eof)
 
     def collect_output(self):
         """Collect output from the process without blocking
@@ -921,8 +1088,7 @@ class SSHClientProcess(SSHProcess, SSHClientStreamSession):
                 self._collect_output(EXTENDED_DATA_STDERR))
 
     # pylint: disable=redefined-builtin
-    @asyncio.coroutine
-    def communicate(self, input=None):
+    async def communicate(self, input=None):
         """Send input to and/or collect output from the process
 
            This method is a coroutine which optionally provides input
@@ -945,7 +1111,7 @@ class SSHClientProcess(SSHProcess, SSHClientStreamSession):
             self._chan.write(input)
             self._chan.write_eof()
 
-        yield from self._chan.wait_closed()
+        await self._chan.wait_closed()
 
         return self.collect_output()
     # pylint: enable=redefined-builtin
@@ -1019,8 +1185,7 @@ class SSHClientProcess(SSHProcess, SSHClientStreamSession):
 
         self._chan.kill()
 
-    @asyncio.coroutine
-    def wait(self, check=False):
+    async def wait(self, check=False, timeout=None):
         """Wait for process to exit
 
            This method is a coroutine which waits for the process to
@@ -1032,27 +1197,47 @@ class SSHClientProcess(SSHProcess, SSHClientStreamSession):
            status from the process with trigger the :exc:`ProcessError`
            exception to be raised.
 
+           If a timeout is specified and it expires before the process
+           exits, the :exc:`TimeoutError` exception will be raised. By
+           default, no timeout is set and this call will wait indefinitely.
+
            :param check:
                Whether or not to raise an error on non-zero exit status
+           :param timeout:
+               Amount of time in seconds to wait for process to exit, or
+               `None` to wait indefinitely
            :type check: `bool`
+           :type timeout: `int`, `float`, or `None`
 
            :returns: :class:`SSHCompletedProcess`
 
-           :raises: :exc:`ProcessError` if check is set to `True`
-                    and the process returns a non-zero exit status
+           :raises: | :exc:`ProcessError` if check is set to `True`
+                      and the process returns a non-zero exit status
+                    | :exc:`TimeoutError` if the timeout expires
+                      before the process exits
 
         """
 
-        stdout_data, stderr_data = yield from self.communicate()
+        try:
+            stdout_data, stderr_data = \
+                await asyncio.wait_for(self.communicate(), timeout)
+        except asyncio.TimeoutError:
+            stdout_data, stderr_data = self.collect_output()
+
+            raise TimeoutError(self.env, self.command, self.subsystem,
+                               self.exit_status, self.exit_signal,
+                               self.returncode, stdout_data,
+                               stderr_data) from None
 
         if check and self.exit_status:
             raise ProcessError(self.env, self.command, self.subsystem,
                                self.exit_status, self.exit_signal,
-                               stdout_data, stderr_data)
+                               self.returncode, stdout_data, stderr_data)
         else:
             return SSHCompletedProcess(self.env, self.command, self.subsystem,
                                        self.exit_status, self.exit_signal,
-                                       stdout_data, stderr_data)
+                                       self.returncode, stdout_data,
+                                       stderr_data)
 
 
 class SSHServerProcess(SSHProcess, SSHServerStreamSession):
@@ -1092,9 +1277,8 @@ class SSHServerProcess(SSHProcess, SSHServerStreamSession):
 
         return self._stderr
 
-    @asyncio.coroutine
-    def redirect(self, stdin=None, stdout=None, stderr=None,
-                 bufsize=io.DEFAULT_BUFFER_SIZE, send_eof=True):
+    async def redirect(self, stdin=None, stdout=None, stderr=None,
+                       bufsize=io.DEFAULT_BUFFER_SIZE, send_eof=True):
         """Perform I/O redirection for the process
 
            This method redirects data going to or from any or all of
@@ -1104,20 +1288,24 @@ class SSHServerProcess(SSHProcess, SSHServerStreamSession):
            The `stdin` argument can be any of the following:
 
                * An :class:`SSHWriter` object
+               * An :class:`asyncio.StreamWriter` object
                * A file object open for write
                * An `int` file descriptor open for write
                * A connected socket object
-               * A string containing the name of a file or device to open
+               * A string or :class:`PurePath <pathlib.PurePath>` containing
+                 the name of a file or device to open
                * `DEVNULL` to discard standard error output
                * `PIPE` to interactively read standard error output
 
            The `stdout` and `stderr` arguments can be any of the following:
 
                * An :class:`SSHReader` object
+               * An :class:`asyncio.StreamReader` object
                * A file object open for read
                * An `int` file descriptor open for read
                * A connected socket object
-               * A string containing the name of a file or device to open
+               * A string or :class:`PurePath <pathlib.PurePath>` containing
+                 the name of a file or device to open
                * `DEVNULL` to provide no input to standard input
                * `PIPE` to interactively write standard input
 
@@ -1127,6 +1315,10 @@ class SSHServerProcess(SSHProcess, SSHServerStreamSession):
            The default value of `None` means to not change redirection
            for that stream.
 
+           .. note:: When passing in asyncio streams, it is the responsibility
+                     of the caller to close the associated transport when it
+                     is no longer needed.
+
            :param stdin:
                Target to feed data from standard input to
            :param stdout:
@@ -1145,50 +1337,32 @@ class SSHServerProcess(SSHProcess, SSHServerStreamSession):
         """
 
         if stdin:
-            yield from self._create_writer(stdin, bufsize, send_eof)
+            await self._create_writer(stdin, bufsize, send_eof)
 
         if stdout:
-            yield from self._create_reader(stdout, bufsize, send_eof)
+            await self._create_reader(stdout, bufsize, send_eof)
 
         if stderr:
-            yield from self._create_reader(stderr, bufsize, send_eof,
-                                           EXTENDED_DATA_STDERR)
+            await self._create_reader(stderr, bufsize, send_eof,
+                                      EXTENDED_DATA_STDERR)
 
-    @asyncio.coroutine
-    def redirect_stdin(self, target, bufsize=io.DEFAULT_BUFFER_SIZE,
-                       send_eof=True):
+    async def redirect_stdin(self, target, bufsize=io.DEFAULT_BUFFER_SIZE,
+                             send_eof=True):
         """Redirect standard input of the process"""
 
-        yield from self.redirect(target, None, None, bufsize, send_eof)
+        await self.redirect(target, None, None, bufsize, send_eof)
 
-    @asyncio.coroutine
-    def redirect_stdout(self, source, bufsize=io.DEFAULT_BUFFER_SIZE,
-                        send_eof=True):
+    async def redirect_stdout(self, source, bufsize=io.DEFAULT_BUFFER_SIZE,
+                              send_eof=True):
         """Redirect standard output of the process"""
 
-        yield from self.redirect(None, source, None, bufsize, send_eof)
+        await self.redirect(None, source, None, bufsize, send_eof)
 
-    @asyncio.coroutine
-    def redirect_stderr(self, source, bufsize=io.DEFAULT_BUFFER_SIZE,
-                        send_eof=True):
+    async def redirect_stderr(self, source, bufsize=io.DEFAULT_BUFFER_SIZE,
+                              send_eof=True):
         """Redirect standard error of the process"""
 
-        yield from self.redirect(None, None, source, bufsize, send_eof)
-
-    def get_environment(self):
-        """Return the environment set by the client (deprecated)"""
-
-        return self.env # pragma: no cover
-
-    def get_command(self):
-        """Return the command the client requested to execute (deprecated)"""
-
-        return self.command # pragma: no cover
-
-    def get_subsystem(self):
-        """Return the subsystem the client requested to open (deprecated)"""
-
-        return self.subsystem # pragma: no cover
+        await self.redirect(None, None, source, bufsize, send_eof)
 
     def get_terminal_type(self):
         """Return the terminal type set by the client for the process
