@@ -1,11 +1,19 @@
-# Copyright (c) 2013-2017 by Ron Frederick <ronf@timeheart.net>.
-# All rights reserved.
+# Copyright (c) 2013-2019 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
-# the terms of the Eclipse Public License v1.0 which accompanies this
+# the terms of the Eclipse Public License v2.0 which accompanies this
 # distribution and is available at:
 #
-#     http://www.eclipse.org/legal/epl-v10.html
+#     http://www.eclipse.org/legal/epl-2.0/
+#
+# This program may also be made available under the following secondary
+# licenses when the conditions for such availability set forth in the
+# Eclipse Public License v2.0 are satisfied:
+#
+#    GNU General Public License, Version 2.0, or any later versions of
+#    that license
+#
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 #
 # Contributors:
 #     Ron Frederick - initial implementation, API, and documentation
@@ -15,14 +23,11 @@
 from .asn1 import ASN1DecodeError, ObjectIdentifier, der_encode, der_decode
 from .crypto import RSAPrivateKey, RSAPublicKey
 from .misc import all_ints
-from .packet import MPInt
+from .packet import MPInt, String
 from .public_key import SSHKey, SSHOpenSSHCertificateV01, KeyExportError
 from .public_key import register_public_key_alg, register_certificate_alg
 from .public_key import register_x509_certificate_alg
 
-# Short variable names are used here, matching names in the spec
-# pylint: disable=invalid-name
-
 
 class _RSAKey(SSHKey):
     """Handler for RSA public key encryption"""
@@ -49,11 +54,9 @@ class _RSAKey(SSHKey):
                      self._key.p, self._key.q))
 
     @classmethod
-    def generate(cls, algorithm, *, key_size=2048, exponent=65537):
+    def generate(cls, _algorithm, *, key_size=2048, exponent=65537):
         """Generate a new RSA private key"""
 
-        # pylint: disable=unused-argument
-
         return cls(RSAPrivateKey.generate(key_size, exponent))
 
     @classmethod
@@ -186,32 +189,31 @@ class _RSAKey(SSHKey):
         return b''.join((MPInt(self._key.d), MPInt(self._key.iqmp),
                          MPInt(self._key.p), MPInt(self._key.q)))
 
-    def sign_der(self, data, sig_algorithm):
-        """Compute a DER-encoded signature of the specified data"""
-
-        # pylint: disable=unused-argument
+    def sign_ssh(self, data, sig_algorithm):
+        """Compute an SSH-encoded signature of the specified data"""
 
         if not self._key.d:
             raise ValueError('Private key needed for signing')
 
-        return self._key.sign(data, sig_algorithm)
+        return String(self._key.sign(data, sig_algorithm))
 
-    def verify_der(self, data, sig_algorithm, sig):
-        """Verify a DER-encoded signature of the specified data"""
+    def verify_ssh(self, data, sig_algorithm, packet):
+        """Verify an SSH-encoded signature of the specified data"""
 
-        # pylint: disable=unused-argument
+        sig = packet.get_string()
+        packet.check_end()
 
         return self._key.verify(data, sig, sig_algorithm)
 
-    def sign_ssh(self, data, sig_algorithm):
-        """Compute an SSH-encoded signature of the specified data"""
+    def encrypt(self, data, algorithm):
+        """Encrypt a block of data with this key"""
 
-        return self.sign_der(data, sig_algorithm)
+        return self._key.encrypt(data, algorithm)
 
-    def verify_ssh(self, data, sig_algorithm, sig):
-        """Verify an SSH-encoded signature of the specified data"""
+    def decrypt(self, data, algorithm):
+        """Decrypt a block of data with this key"""
 
-        return self.verify_der(data, sig_algorithm, sig)
+        return self._key.decrypt(data, algorithm)
 
 
 register_public_key_alg(b'ssh-rsa', _RSAKey)
