@@ -1,11 +1,19 @@
-# Copyright (c) 2015-2018 by Ron Frederick <ronf@timeheart.net>.
-# All rights reserved.
+# Copyright (c) 2015-2019 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
-# the terms of the Eclipse Public License v1.0 which accompanies this
+# the terms of the Eclipse Public License v2.0 which accompanies this
 # distribution and is available at:
 #
-#     http://www.eclipse.org/legal/epl-v10.html
+#     http://www.eclipse.org/legal/epl-2.0/
+#
+# This program may also be made available under the following secondary
+# licenses when the conditions for such availability set forth in the
+# Eclipse Public License v2.0 are satisfied:
+#
+#    GNU General Public License, Version 2.0, or any later versions of
+#    that license
+#
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 #
 # Contributors:
 #     Ron Frederick - initial implementation, API, and documentation
@@ -19,23 +27,24 @@ from hashlib import sha1
 
 import asyncssh
 
-from asyncssh.dh import MSG_KEXDH_INIT, MSG_KEXDH_REPLY
-from asyncssh.dh import _KexDHGex, MSG_KEX_DH_GEX_REQUEST, MSG_KEX_DH_GEX_GROUP
-from asyncssh.dh import MSG_KEX_DH_GEX_INIT, MSG_KEX_DH_GEX_REPLY
-from asyncssh.dh import MSG_KEXGSS_INIT, MSG_KEXGSS_COMPLETE, MSG_KEXGSS_ERROR
-from asyncssh.ecdh import MSG_KEX_ECDH_INIT, MSG_KEX_ECDH_REPLY
+from asyncssh.crypto import curve25519_available, curve448_available
+from asyncssh.crypto import Curve25519DH, Curve448DH, ECDH
+from asyncssh.kex_dh import MSG_KEXDH_INIT, MSG_KEXDH_REPLY
+from asyncssh.kex_dh import MSG_KEX_DH_GEX_REQUEST, MSG_KEX_DH_GEX_GROUP
+from asyncssh.kex_dh import MSG_KEX_DH_GEX_INIT, MSG_KEX_DH_GEX_REPLY, _KexDHGex
+from asyncssh.kex_dh import MSG_KEX_ECDH_INIT, MSG_KEX_ECDH_REPLY
+from asyncssh.kex_dh import MSG_KEXGSS_INIT, MSG_KEXGSS_COMPLETE
+from asyncssh.kex_dh import MSG_KEXGSS_ERROR
+from asyncssh.kex_rsa import MSG_KEXRSA_PUBKEY, MSG_KEXRSA_SECRET
+from asyncssh.kex_rsa import MSG_KEXRSA_DONE
 from asyncssh.gss import GSSClient, GSSServer
 from asyncssh.kex import register_kex_alg, get_kex_algs, get_kex
-from asyncssh.misc import DisconnectError
 from asyncssh.packet import SSHPacket, Boolean, Byte, MPInt, String
-from asyncssh.public_key import SSHLocalKeyPair, decode_ssh_public_key
+from asyncssh.public_key import decode_ssh_public_key
 
 from .util import asynctest, gss_available, patch_gss
 from .util import AsyncTestCase, ConnectionStub
 
-# Short variable names are used here, matching names in the specs
-# pylint: disable=invalid-name
-
 
 class _KexConnectionStub(ConnectionStub):
     """Connection stub class to test key exchange"""
@@ -48,8 +57,10 @@ class _KexConnectionStub(ConnectionStub):
 
         self._kex = get_kex(self, alg)
 
-        if self.is_client():
-            self._kex.start()
+    def start(self):
+        """Start key exchange"""
+
+        self._kex.start()
 
     def connection_lost(self, exc):
         """Handle the closing of a connection"""
@@ -59,8 +70,6 @@ class _KexConnectionStub(ConnectionStub):
     def enable_gss_kex_auth(self):
         """Ignore request to enable GSS key exchange authentication"""
 
-        pass
-
     def process_packet(self, data):
         """Process an incoming packet"""
 
@@ -80,10 +89,10 @@ class _KexConnectionStub(ConnectionStub):
 
         self._key_waiter.set_result(self._kex.compute_key(k, h, b'A', h, 128))
 
-    def get_key(self):
+    async def get_key(self):
         """Return generated key data"""
 
-        return (yield from self._key_waiter)
+        return await self._key_waiter
 
     def get_gss_context(self):
         """Return the GSS context associated with this connection"""
@@ -137,6 +146,22 @@ class _KexConnectionStub(ConnectionStub):
                                       String(host_key_data),
                                       String(server_pub), String(sig))))
 
+    def simulate_rsa_pubkey(self, host_key_data, trans_key_data):
+        """Simulate receiving an RSA pubkey packet"""
+
+        self.process_packet(Byte(MSG_KEXRSA_PUBKEY) + String(host_key_data) +
+                            String(trans_key_data))
+
+    def simulate_rsa_secret(self, encrypted_k):
+        """Simulate receiving an RSA secret packet"""
+
+        self.process_packet(Byte(MSG_KEXRSA_SECRET) + String(encrypted_k))
+
+    def simulate_rsa_done(self, sig):
+        """Simulate receiving an RSA done packet"""
+
+        self.process_packet(Byte(MSG_KEXRSA_DONE) + String(sig))
+
 
 class _KexClientStub(_KexConnectionStub):
     """Stub class for client connection"""
@@ -151,16 +176,12 @@ class _KexClientStub(_KexConnectionStub):
     def __init__(self, alg, gss_host):
         server_conn = _KexServerStub(alg, gss_host, self)
 
-        try:
-            if gss_host:
-                gss = GSSClient(gss_host, 'delegate' in gss_host)
-            else:
-                gss = None
+        if gss_host:
+            gss = GSSClient(gss_host, 'delegate' in gss_host)
+        else:
+            gss = None
 
-            super().__init__(alg, gss, server_conn)
-        except DisconnectError:
-            server_conn.close()
-            raise
+        super().__init__(alg, gss, server_conn)
 
     def connection_lost(self, exc):
         """Handle the closing of a connection"""
@@ -189,7 +210,7 @@ class _KexServerStub(_KexConnectionStub):
             self._server_host_key = None
         else:
             priv_key = asyncssh.generate_private_key('ssh-rsa')
-            self._server_host_key = SSHLocalKeyPair(priv_key)
+            self._server_host_key = asyncssh.load_keypairs(priv_key)[0]
 
     def connection_lost(self, exc):
         """Handle the closing of a connection"""
@@ -209,51 +230,49 @@ class _KexServerStub(_KexConnectionStub):
 class _TestKex(AsyncTestCase):
     """Unit tests for kex module"""
 
-    @asyncio.coroutine
-    def _check_kex(self, alg, gss_host=None):
+    async def _check_kex(self, alg, gss_host=None):
         """Unit test key exchange"""
 
         client_conn, server_conn = _KexClientStub.make_pair(alg, gss_host)
 
         try:
-            self.assertEqual((yield from client_conn.get_key()),
-                             (yield from server_conn.get_key()))
+            client_conn.start()
+            server_conn.start()
+
+            self.assertEqual((await client_conn.get_key()),
+                             (await server_conn.get_key()))
         finally:
             client_conn.close()
             server_conn.close()
 
     @asynctest
-    def test_key_exchange_algs(self):
+    async def test_key_exchange_algs(self):
         """Unit test key exchange algorithms"""
 
         for alg in get_kex_algs():
             with self.subTest(alg=alg):
                 if alg.startswith(b'gss-'):
                     if gss_available: # pragma: no branch
-                        yield from self._check_kex(alg + b'-mech', '1')
+                        await self._check_kex(alg + b'-mech', '1')
                 else:
-                    yield from self._check_kex(alg)
+                    await self._check_kex(alg)
 
         if gss_available: # pragma: no branch
             for steps in range(4):
                 with self.subTest('GSS key exchange', steps=steps):
-                    yield from self._check_kex(b'gss-group1-sha1-mech',
-                                               str(steps))
+                    await self._check_kex(b'gss-group1-sha1-mech', str(steps))
 
             with self.subTest('GSS with credential delegation'):
-                yield from self._check_kex(b'gss-group1-sha1-mech',
-                                           '1,delegate')
+                await self._check_kex(b'gss-group1-sha1-mech', '1,delegate')
 
             with self.subTest('GSS with no host key'):
-                yield from self._check_kex(b'gss-group1-sha1-mech',
-                                           '1,no_host_key')
+                await self._check_kex(b'gss-group1-sha1-mech', '1,no_host_key')
 
             with self.subTest('GSS with full host principal'):
-                yield from self._check_kex(b'gss-group1-sha1-mech',
-                                           'host/1@TEST')
+                await self._check_kex(b'gss-group1-sha1-mech', 'host/1@TEST')
 
     @asynctest
-    def test_dh_gex_old(self):
+    async def test_dh_gex_old(self):
         """Unit test old DH group exchange request"""
 
         register_kex_alg(b'dh-gex-sha1-1024', _KexDHGex, sha1, 1024)
@@ -261,10 +280,10 @@ class _TestKex(AsyncTestCase):
 
         for size in (b'1024', b'2048'):
             with self.subTest('Old DH group exchange', size=size):
-                yield from self._check_kex(b'dh-gex-sha1-' + size)
+                await self._check_kex(b'dh-gex-sha1-' + size)
 
     @asynctest
-    def test_dh_gex(self):
+    async def test_dh_gex(self):
         """Unit test old DH group exchange request"""
 
         register_kex_alg(b'dh-gex-sha1-1024-1536', _KexDHGex, sha1, 1024, 1536)
@@ -276,10 +295,10 @@ class _TestKex(AsyncTestCase):
         for size in (b'1024-1536', b'1536-3072', b'2560-2560',
                      b'2560-4096', b'9216-9216'):
             with self.subTest('Old DH group exchange', size=size):
-                yield from self._check_kex(b'dh-gex-sha1-' + size)
+                await self._check_kex(b'dh-gex-sha1-' + size)
 
     @asynctest
-    def test_dh_errors(self):
+    async def test_dh_errors(self):
         """Unit test error conditions in DH key exchange"""
 
         client_conn, server_conn = \
@@ -288,57 +307,59 @@ class _TestKex(AsyncTestCase):
         host_key = server_conn.get_server_host_key()
 
         with self.subTest('Init sent to client'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.ProtocolError):
                 client_conn.process_packet(Byte(MSG_KEXDH_INIT))
 
         with self.subTest('Reply sent to server'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.ProtocolError):
                 server_conn.process_packet(Byte(MSG_KEXDH_REPLY))
 
         with self.subTest('Invalid e value'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.ProtocolError):
                 server_conn.simulate_dh_init(0)
 
         with self.subTest('Invalid f value'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.ProtocolError):
+                client_conn.start()
                 client_conn.simulate_dh_reply(host_key.public_data, 0, b'')
 
         with self.subTest('Invalid signature'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.KeyExchangeFailed):
+                client_conn.start()
                 client_conn.simulate_dh_reply(host_key.public_data, 1, b'')
 
         client_conn.close()
         server_conn.close()
 
     @asynctest
-    def test_dh_gex_errors(self):
+    async def test_dh_gex_errors(self):
         """Unit test error conditions in DH group exchange"""
 
         client_conn, server_conn = \
             _KexClientStub.make_pair(b'diffie-hellman-group-exchange-sha1')
 
         with self.subTest('Request sent to client'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.ProtocolError):
                 client_conn.process_packet(Byte(MSG_KEX_DH_GEX_REQUEST))
 
         with self.subTest('Group sent to server'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.ProtocolError):
                 server_conn.simulate_dh_gex_group(1, 2)
 
         with self.subTest('Init sent to client'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.ProtocolError):
                 client_conn.simulate_dh_gex_init(1)
 
         with self.subTest('Init sent before group'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.ProtocolError):
                 server_conn.simulate_dh_gex_init(1)
 
         with self.subTest('Reply sent to server'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.ProtocolError):
                 server_conn.simulate_dh_gex_reply(b'', 1, b'')
 
         with self.subTest('Reply sent before group'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.ProtocolError):
                 client_conn.simulate_dh_gex_reply(b'', 1, b'')
 
         client_conn.close()
@@ -346,106 +367,99 @@ class _TestKex(AsyncTestCase):
 
     @unittest.skipUnless(gss_available, 'GSS not available')
     @asynctest
-    def test_gss_errors(self):
+    async def test_gss_errors(self):
         """Unit test error conditions in GSS key exchange"""
 
         client_conn, server_conn = \
             _KexClientStub.make_pair(b'gss-group1-sha1-mech', '3')
 
         with self.subTest('Init sent to client'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.ProtocolError):
                 client_conn.process_packet(Byte(MSG_KEXGSS_INIT))
 
         with self.subTest('Complete sent to server'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.ProtocolError):
                 server_conn.process_packet(Byte(MSG_KEXGSS_COMPLETE))
 
         with self.subTest('Exchange failed to complete'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.ProtocolError):
                 client_conn.simulate_gss_complete(1, b'succeed')
 
         with self.subTest('Error sent to server'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.ProtocolError):
                 server_conn.process_packet(Byte(MSG_KEXGSS_ERROR))
 
         client_conn.close()
         server_conn.close()
 
         with self.subTest('Signature verification failure'):
-            with self.assertRaises(DisconnectError):
-                yield from self._check_kex(b'gss-group1-sha1-mech', '0,fail')
+            with self.assertRaises(asyncssh.KeyExchangeFailed):
+                await self._check_kex(b'gss-group1-sha1-mech', '0,fail')
 
         with self.subTest('Empty token in init'):
-            with self.assertRaises(DisconnectError):
-                yield from self._check_kex(b'gss-group1-sha1-mech',
-                                           '0,empty_init')
+            with self.assertRaises(asyncssh.ProtocolError):
+                await self._check_kex(b'gss-group1-sha1-mech', '0,empty_init')
 
         with self.subTest('Empty token in continue'):
-            with self.assertRaises(DisconnectError):
-                yield from self._check_kex(b'gss-group1-sha1-mech',
-                                           '1,empty_continue')
+            with self.assertRaises(asyncssh.ProtocolError):
+                await self._check_kex(b'gss-group1-sha1-mech',
+                                      '1,empty_continue')
 
         with self.subTest('Token after complete'):
-            with self.assertRaises(DisconnectError):
-                yield from self._check_kex(b'gss-group1-sha1-mech',
-                                           '0,continue_token')
+            with self.assertRaises(asyncssh.ProtocolError):
+                await self._check_kex(b'gss-group1-sha1-mech',
+                                      '0,continue_token')
 
         for steps in range(2):
             with self.subTest('Token after complete', steps=steps):
-                with self.assertRaises(DisconnectError):
-                    yield from self._check_kex(b'gss-group1-sha1-mech',
-                                               str(steps) + ',extra_token')
+                with self.assertRaises(asyncssh.ProtocolError):
+                    await self._check_kex(b'gss-group1-sha1-mech',
+                                          str(steps) + ',extra_token')
 
         with self.subTest('Context not secure'):
-            with self.assertRaises(DisconnectError):
-                yield from self._check_kex(b'gss-group1-sha1-mech',
-                                           '1,no_server_integrity')
+            with self.assertRaises(asyncssh.ProtocolError):
+                await self._check_kex(b'gss-group1-sha1-mech',
+                                      '1,no_server_integrity')
 
         with self.subTest('GSS error'):
-            with self.assertRaises(DisconnectError):
-                yield from self._check_kex(b'gss-group1-sha1-mech',
-                                           '1,step_error')
+            with self.assertRaises(asyncssh.KeyExchangeFailed):
+                await self._check_kex(b'gss-group1-sha1-mech', '1,step_error')
 
         with self.subTest('GSS error with error token'):
-            with self.assertRaises(DisconnectError):
-                yield from self._check_kex(b'gss-group1-sha1-mech',
-                                           '1,step_error,errtok')
+            with self.assertRaises(asyncssh.KeyExchangeFailed):
+                await self._check_kex(b'gss-group1-sha1-mech',
+                                      '1,step_error,errtok')
 
     @asynctest
-    def test_ecdh_errors(self):
+    async def test_ecdh_errors(self):
         """Unit test error conditions in ECDH key exchange"""
 
-        try:
-            from asyncssh.crypto import ECDH
-        except ImportError: # pragma: no cover
-            return
-
         client_conn, server_conn = \
             _KexClientStub.make_pair(b'ecdh-sha2-nistp256')
 
         with self.subTest('Init sent to client'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.ProtocolError):
                 client_conn.simulate_ecdh_init(b'')
 
         with self.subTest('Invalid client public key'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.ProtocolError):
                 server_conn.simulate_ecdh_init(b'')
 
         with self.subTest('Reply sent to server'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.ProtocolError):
                 server_conn.simulate_ecdh_reply(b'', b'', b'')
 
         with self.subTest('Invalid server host key'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.KeyImportError):
                 client_conn.simulate_ecdh_reply(b'', b'', b'')
 
         with self.subTest('Invalid server public key'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.ProtocolError):
                 host_key = server_conn.get_server_host_key()
                 client_conn.simulate_ecdh_reply(host_key.public_data, b'', b'')
 
         with self.subTest('Invalid signature'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.KeyExchangeFailed):
                 host_key = server_conn.get_server_host_key()
                 server_pub = ECDH(b'nistp256').get_public()
                 client_conn.simulate_ecdh_reply(host_key.public_data,
@@ -454,29 +468,32 @@ class _TestKex(AsyncTestCase):
         client_conn.close()
         server_conn.close()
 
+    @unittest.skipUnless(curve25519_available, 'Curve25519 not available')
     @asynctest
-    def test_curve25519dh_errors(self):
+    async def test_curve25519dh_errors(self):
         """Unit test error conditions in Curve25519DH key exchange"""
 
-        try:
-            from asyncssh.crypto import Curve25519DH
-        except ImportError: # pragma: no cover
-            return
-
         client_conn, server_conn = \
-            _KexClientStub.make_pair(b'curve25519-sha256@libssh.org')
+            _KexClientStub.make_pair(b'curve25519-sha256')
 
         with self.subTest('Invalid client public key'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.ProtocolError):
                 server_conn.simulate_ecdh_init(b'')
 
         with self.subTest('Invalid server public key'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.ProtocolError):
                 host_key = server_conn.get_server_host_key()
                 client_conn.simulate_ecdh_reply(host_key.public_data, b'', b'')
 
+        with self.subTest('Invalid peer public key'):
+            with self.assertRaises(asyncssh.ProtocolError):
+                host_key = server_conn.get_server_host_key()
+                server_pub = b'\x01' + 31*b'\x00'
+                client_conn.simulate_ecdh_reply(host_key.public_data,
+                                                server_pub, b'')
+
         with self.subTest('Invalid signature'):
-            with self.assertRaises(DisconnectError):
+            with self.assertRaises(asyncssh.KeyExchangeFailed):
                 host_key = server_conn.get_server_host_key()
                 server_pub = Curve25519DH().get_public()
                 client_conn.simulate_ecdh_reply(host_key.public_data,
@@ -484,3 +501,76 @@ class _TestKex(AsyncTestCase):
 
         client_conn.close()
         server_conn.close()
+
+    @unittest.skipUnless(curve448_available, 'Curve448 not available')
+    @asynctest
+    async def test_curve448dh_errors(self):
+        """Unit test error conditions in Curve448DH key exchange"""
+
+        client_conn, server_conn = \
+            _KexClientStub.make_pair(b'curve448-sha512')
+
+        with self.subTest('Invalid client public key'):
+            with self.assertRaises(asyncssh.ProtocolError):
+                server_conn.simulate_ecdh_init(b'')
+
+        with self.subTest('Invalid server public key'):
+            with self.assertRaises(asyncssh.ProtocolError):
+                host_key = server_conn.get_server_host_key()
+                client_conn.simulate_ecdh_reply(host_key.public_data, b'', b'')
+
+        with self.subTest('Invalid peer public key'):
+            with self.assertRaises(asyncssh.ProtocolError):
+                host_key = server_conn.get_server_host_key()
+                server_pub = b'\x01' + 55*b'\x00'
+                client_conn.simulate_ecdh_reply(host_key.public_data,
+                                                server_pub, b'')
+
+        with self.subTest('Invalid signature'):
+            with self.assertRaises(asyncssh.KeyExchangeFailed):
+                host_key = server_conn.get_server_host_key()
+                server_pub = Curve448DH().get_public()
+                client_conn.simulate_ecdh_reply(host_key.public_data,
+                                                server_pub, b'')
+
+        client_conn.close()
+        server_conn.close()
+
+    @asynctest
+    async def test_rsa_errors(self):
+        """Unit test error conditions in RSA key exchange"""
+
+        client_conn, server_conn = \
+            _KexClientStub.make_pair(b'rsa2048-sha256')
+
+        with self.subTest('Pubkey sent to server'):
+            with self.assertRaises(asyncssh.ProtocolError):
+                server_conn.simulate_rsa_pubkey(b'', b'')
+
+        with self.subTest('Secret sent to client'):
+            with self.assertRaises(asyncssh.ProtocolError):
+                client_conn.simulate_rsa_secret(b'')
+
+        with self.subTest('Done sent to server'):
+            with self.assertRaises(asyncssh.ProtocolError):
+                server_conn.simulate_rsa_done(b'')
+
+        with self.subTest('Invalid transient public key'):
+            with self.assertRaises(asyncssh.ProtocolError):
+                client_conn.simulate_rsa_pubkey(b'', b'')
+
+        with self.subTest('Invalid encrypted secret'):
+            with self.assertRaises(asyncssh.KeyExchangeFailed):
+                server_conn.start()
+                server_conn.simulate_rsa_secret(b'')
+
+        with self.subTest('Invalid signature'):
+            with self.assertRaises(asyncssh.KeyExchangeFailed):
+                host_key = server_conn.get_server_host_key()
+                trans_key = asyncssh.generate_private_key('ssh-rsa', 2048)
+                client_conn.simulate_rsa_pubkey(host_key.public_data,
+                                                trans_key.public_data)
+                client_conn.simulate_rsa_done(b'')
+
+        client_conn.close()
+        server_conn.close()
