@@ -1,11 +1,19 @@
-# Copyright (c) 2015-2018 by Ron Frederick <ronf@timeheart.net>.
-# All rights reserved.
+# Copyright (c) 2015-2019 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
-# the terms of the Eclipse Public License v1.0 which accompanies this
+# the terms of the Eclipse Public License v2.0 which accompanies this
 # distribution and is available at:
 #
-#     http://www.eclipse.org/legal/epl-v10.html
+#     http://www.eclipse.org/legal/epl-2.0/
+#
+# This program may also be made available under the following secondary
+# licenses when the conditions for such availability set forth in the
+# Eclipse Public License v2.0 are satisfied:
+#
+#    GNU General Public License, Version 2.0, or any later versions of
+#    that license
+#
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 #
 # Contributors:
 #     Ron Frederick - initial implementation, API, and documentation
@@ -31,12 +39,6 @@ try:
 except ImportError: # pragma: no cover
     bcrypt_available = False
 
-try:
-    import libnacl
-    libnacl_available = True
-except (ImportError, OSError, AttributeError): # pragma: no cover
-    libnacl_available = False
-
 try:
     import uvloop
     uvloop_available = True
@@ -51,37 +53,40 @@ except ImportError: # pragma: no cover
 
 # pylint: enable=unused-import
 
-from asyncssh.constants import DISC_CONNECTION_LOST
 from asyncssh.gss import gss_available
 from asyncssh.logging import logger
-from asyncssh.misc import DisconnectError, SignalReceived, create_task
+from asyncssh.misc import ConnectionLost, SignalReceived
 from asyncssh.packet import Byte, String, UInt32, UInt64
 
 
-def asynctest(func):
+def asynctest(coro):
     """Decorator for async tests, for use with AsyncTestCase"""
 
-    @functools.wraps(func)
+    @functools.wraps(coro)
     def async_wrapper(self, *args, **kwargs):
-        """Run a function as a coroutine and wait for it to finish"""
+        """Run a coroutine and wait for it to finish"""
 
-        wrapped_func = asyncio.coroutine(func)(self, *args, **kwargs)
-        return self.loop.run_until_complete(wrapped_func)
+        return self.loop.run_until_complete(coro(self, *args, **kwargs))
 
     return async_wrapper
 
 
-def asynctest35(func):
-    """Decorator for Python 3.5 async tests, for use with AsyncTestCase"""
+def patch_getnameinfo(cls):
+    """Decorator for patching socket.getnameinfo"""
 
-    @functools.wraps(func)
-    def async_wrapper(self, *args, **kwargs):
-        """Run a function as a coroutine and wait for it to finish"""
+    def getnameinfo(sockaddr, flags):
+        """Mock reverse DNS lookup of client address"""
 
-        wrapped_func = func(self, *args, **kwargs)
-        return self.loop.run_until_complete(wrapped_func)
+        # pylint: disable=unused-argument
 
-    return async_wrapper
+        host, port = sockaddr
+
+        if host == '127.0.0.1':
+            return ('localhost', port)
+        else:
+            return sockaddr
+
+    return patch('socket.getnameinfo', getnameinfo)(cls)
 
 
 def patch_gss(cls):
@@ -90,6 +95,8 @@ def patch_gss(cls):
     if not gss_available: # pragma: no cover
         return cls
 
+    # pylint: disable=import-outside-toplevel
+
     if sys.platform == 'win32': # pragma: no cover
         from .sspi_stub import SSPIAuth
 
@@ -107,13 +114,12 @@ def patch_gss(cls):
     return cls
 
 
-@asyncio.coroutine
-def echo(stdin, stdout, stderr=None):
+async def echo(stdin, stdout, stderr=None):
     """Echo data from stdin back to stdout and stderr (if open)"""
 
     try:
         while not stdin.at_eof():
-            data = yield from stdin.read(65536)
+            data = await stdin.read(65536)
 
             if data:
                 stdout.write(data)
@@ -121,15 +127,15 @@ def echo(stdin, stdout, stderr=None):
                 if stderr:
                     stderr.write(data)
 
-        yield from stdout.drain()
+        await stdout.drain()
 
         if stderr:
-            yield from stderr.drain()
+            await stderr.drain()
 
         stdout.write_eof()
     except SignalReceived as exc:
         if exc.signal == 'ABRT':
-            raise DisconnectError(DISC_CONNECTION_LOST, 'Abort')
+            raise ConnectionLost('Abort')
         else:
             stdin.channel.exit_with_signal(exc.signal)
     except OSError:
@@ -205,13 +211,12 @@ class ConnectionStub:
 
         return self._logger
 
-    @asyncio.coroutine
-    def _run_task(self, coro):
+    async def _run_task(self, coro):
         """Run an asynchronous task"""
 
         # pylint: disable=broad-except
         try:
-            yield from coro
+            await coro
         except Exception as exc:
             if self._peer: # pragma: no branch
                 self.queue_packet(exc)
@@ -221,7 +226,7 @@ class ConnectionStub:
     def create_task(self, coro):
         """Create an asynchronous task"""
 
-        return create_task(self._run_task(coro))
+        return asyncio.ensure_future(self._run_task(coro))
 
     def is_client(self):
         """Return if this is a client connection"""
@@ -238,12 +243,11 @@ class ConnectionStub:
 
         return self._peer
 
-    @asyncio.coroutine
-    def _process_packets(self):
+    async def _process_packets(self):
         """Process the queue of incoming packets"""
 
         while True:
-            data = yield from self._packet_queue.get()
+            data = await self._packet_queue.get()
 
             if data is None or isinstance(data, Exception):
                 self._queue_task = None
@@ -287,10 +291,37 @@ class ConnectionStub:
             self._queue_task = None
 
 
-class TempDirTestCase(unittest.TestCase):
+if hasattr(unittest.TestCase, 'addClassCleanup'):
+    ClassCleanupTestCase = unittest.TestCase
+else: # pragma: no cover
+    class ClassCleanupTestCase(unittest.TestCase):
+        """Stripped down version of class cleanup for Python 3.7 & earlier"""
+
+        _class_cleanups = []
+
+        # pylint: disable=arguments-differ
+
+        @classmethod
+        def addClassCleanup(cls, function, *args, **kwargs):
+            """Add a cleanup to run after tearDownClass"""
+
+            cls._class_cleanups.append((function, args, kwargs))
+
+        @classmethod
+        def tearDownClass(cls):
+            """Run cleanups after tearDown"""
+
+            super().tearDownClass()
+
+            while cls._class_cleanups:
+                function, args, kwargs = cls._class_cleanups.pop()
+                function(*args, **kwargs)
+
+
+class TempDirTestCase(ClassCleanupTestCase):
     """Unit test class which operates in a temporary directory"""
 
-    tempdir = None
+    _tempdir = None
 
     @classmethod
     def setUpClass(cls):
@@ -326,7 +357,6 @@ class AsyncTestCase(TempDirTestCase):
         asyncio.set_event_loop(cls.loop)
 
         try:
-            # pylint: disable=no-member
             cls.loop.run_until_complete(cls.asyncSetUpClass())
         except AttributeError:
             pass
@@ -336,7 +366,6 @@ class AsyncTestCase(TempDirTestCase):
         """Run async class teardown and close event loop"""
 
         try:
-            # pylint: disable=no-member
             cls.loop.run_until_complete(cls.asyncTearDownClass())
         except AttributeError:
             pass
@@ -349,7 +378,6 @@ class AsyncTestCase(TempDirTestCase):
         """Run async setup if any"""
 
         try:
-            # pylint: disable=no-member
             self.loop.run_until_complete(self.asyncSetUp())
         except AttributeError:
             pass
@@ -358,7 +386,6 @@ class AsyncTestCase(TempDirTestCase):
         """Run async teardown if any"""
 
         try:
-            # pylint: disable=no-member
             self.loop.run_until_complete(self.asyncTearDown())
         except AttributeError:
             pass
